"""Pydantic schemas for Raids endpoints."""

from datetime import datetime

from .base import TwitchBaseModel


class StartRaidRequest(TwitchBaseModel):
    """Request to start a raid."""

    from_broadcaster_id: str
    to_broadcaster_id: str


class StartRaidResponse(TwitchBaseModel):
    """Response from starting a raid."""

    created_at: datetime
    is_mature: bool


class CancelRaidRequest(TwitchBaseModel):
    """Request to cancel a raid."""

    broadcaster_id: str



"""Pydantic schemas for Users endpoints."""

from datetime import datetime

from pydantic import Field

from .base import TwitchBaseModel


class User(TwitchBaseModel):
    """User data from Twitch API."""

    id: str
    login: str
    display_name: str
    type: str = ""  # staff, admin, global_mod, or empty
    broadcaster_type: str = ""  # partner, affiliate, or empty
    description: str = ""
    profile_image_url: str = ""
    offline_image_url: str = ""
    email: str | None = None
    created_at: datetime


class GetUsersRequest(TwitchBaseModel):
    """Request params for Get Users endpoint."""

    id: list[str] | None = None
    login: list[str] | None = None


class UpdateUserRequest(TwitchBaseModel):
    """Request params for Update User endpoint."""

    description: str | None = None


class UserBlockTarget(TwitchBaseModel):
    """Blocked user info."""

    user_id: str
    user_login: str
    display_name: str


class GetUserBlockListRequest(TwitchBaseModel):
    """Request params for Get User Block List."""

    broadcaster_id: str
    first: int | None = Field(default=None, le=100)
    after: str | None = None


class BlockUserRequest(TwitchBaseModel):
    """Request params for Block User."""

    target_user_id: str
    source_context: str | None = None  # chat, whisper
    reason: str | None = None  # harassment, spam, other


class UnblockUserRequest(TwitchBaseModel):
    """Request params for Unblock User."""

    target_user_id: str


class UserExtension(TwitchBaseModel):
    """User extension info."""

    id: str
    version: str
    name: str
    can_activate: bool
    type: list[str]


class ActiveExtension(TwitchBaseModel):
    """Active extension configuration."""

    active: bool
    id: str | None = None
    version: str | None = None
    name: str | None = None
    x: int | None = None
    y: int | None = None


class UserActiveExtensions(TwitchBaseModel):
    """User's active extensions."""

    panel: dict[str, ActiveExtension]
    overlay: dict[str, ActiveExtension]
    component: dict[str, ActiveExtension]


class UpdateUserExtensionsRequest(TwitchBaseModel):
    """Request to update active extensions."""

    data: UserActiveExtensions



"""Raids endpoints."""

from typing import TYPE_CHECKING

from twitch_sdk.schemas.base import TwitchResponse
from twitch_sdk.schemas.raids import (
    CancelRaidRequest,
    StartRaidRequest,
    StartRaidResponse,
)

if TYPE_CHECKING:
    from twitch_client import TwitchHTTPClient


async def start_raid(
    client: "TwitchHTTPClient",
    params: StartRaidRequest,
) -> TwitchResponse[StartRaidResponse]:
    """Start a raid to another channel.

    Requires channel:manage:raids scope.
    """
    query = params.model_dump(exclude_none=True)
    response = await client.post("/raids", params=query)
    return TwitchResponse[StartRaidResponse].model_validate(response)


async def cancel_raid(
    client: "TwitchHTTPClient",
    params: CancelRaidRequest,
) -> None:
    """Cancel a pending raid.

    Requires channel:manage:raids scope.
    """
    query = params.model_dump(exclude_none=True)
    await client.delete("/raids", params=query)



"""Ads endpoints."""

from typing import TYPE_CHECKING

from twitch_sdk.schemas.base import TwitchResponse
from twitch_sdk.schemas.ads import (
    AdSchedule,
    GetAdScheduleRequest,
    SnoozeNextAdRequest,
    SnoozeNextAdResponse,
    StartCommercialRequest,
    StartCommercialResponse,
)

if TYPE_CHECKING:
    from twitch_client import TwitchHTTPClient


async def start_commercial(
    client: "TwitchHTTPClient",
    params: StartCommercialRequest,
) -> TwitchResponse[StartCommercialResponse]:
    """Start a commercial on a channel.

    Requires channel:edit:commercial scope.
    """
    data = params.model_dump(exclude_none=True)
    response = await client.post("/channels/commercial", data=data)
    return TwitchResponse[StartCommercialResponse].model_validate(response)


async def get_ad_schedule(
    client: "TwitchHTTPClient",
    params: GetAdScheduleRequest,
) -> TwitchResponse[AdSchedule]:
    """Get ad schedule for a channel.

    Requires channel:read:ads scope.
    """
    query = params.model_dump(exclude_none=True)
    response = await client.get("/channels/ads", params=query)
    return TwitchResponse[AdSchedule].model_validate(response)


async def snooze_next_ad(
    client: "TwitchHTTPClient",
    params: SnoozeNextAdRequest,
) -> TwitchResponse[SnoozeNextAdResponse]:
    """Snooze the next ad break.

    Requires channel:manage:ads scope.
    """
    query = params.model_dump(exclude_none=True)
    response = await client.post("/channels/ads/schedule/snooze", params=query)
    return TwitchResponse[SnoozeNextAdResponse].model_validate(response)



"""Channel Points endpoints."""

from typing import TYPE_CHECKING

from twitch_sdk.schemas.base import TwitchResponse
from twitch_sdk.schemas.channel_points import (
    CreateCustomRewardRequest,
    CustomReward,
    DeleteCustomRewardRequest,
    GetCustomRewardRedemptionRequest,
    GetCustomRewardsRequest,
    RewardRedemption,
    UpdateCustomRewardRequest,
    UpdateRedemptionStatusRequest,
)

if TYPE_CHECKING:
    from twitch_client import TwitchHTTPClient


async def get_custom_rewards(
    client: "TwitchHTTPClient",
    params: GetCustomRewardsRequest,
) -> TwitchResponse[CustomReward]:
    """Get custom rewards for a channel.

    Requires channel:read:redemptions or channel:manage:redemptions scope.
    """
    query = params.model_dump(exclude_none=True)
    response = await client.get("/channel_points/custom_rewards", params=query)
    return TwitchResponse[CustomReward].model_validate(response)


async def create_custom_reward(
    client: "TwitchHTTPClient",
    params: CreateCustomRewardRequest,
) -> TwitchResponse[CustomReward]:
    """Create a custom reward on a channel.

    Requires channel:manage:redemptions scope.
    """
    query = {"broadcaster_id": params.broadcaster_id}
    data = params.model_dump(exclude={"broadcaster_id"}, exclude_none=True)
    response = await client.post("/channel_points/custom_rewards", params=query, data=data)
    return TwitchResponse[CustomReward].model_validate(response)


async def update_custom_reward(
    client: "TwitchHTTPClient",
    params: UpdateCustomRewardRequest,
) -> TwitchResponse[CustomReward]:
    """Update a custom reward on a channel.

    Requires channel:manage:redemptions scope.
    """
    query = {"broadcaster_id": params.broadcaster_id, "id": params.id}
    data = params.model_dump(exclude={"broadcaster_id", "id"}, exclude_none=True)
    response = await client.patch("/channel_points/custom_rewards", params=query, data=data)
    return TwitchResponse[CustomReward].model_validate(response)


async def delete_custom_reward(
    client: "TwitchHTTPClient",
    params: DeleteCustomRewardRequest,
) -> None:
    """Delete a custom reward from a channel.

    Requires channel:manage:redemptions scope.
    """
    query = params.model_dump(exclude_none=True)
    await client.delete("/channel_points/custom_rewards", params=query)


async def get_custom_reward_redemption(
    client: "TwitchHTTPClient",
    params: GetCustomRewardRedemptionRequest,
) -> TwitchResponse[RewardRedemption]:
    """Get redemptions for a custom reward.

    Requires channel:read:redemptions or channel:manage:redemptions scope.
    """
    query = params.model_dump(exclude_none=True)
    response = await client.get("/channel_points/custom_rewards/redemptions", params=query)
    return TwitchResponse[RewardRedemption].model_validate(response)


async def update_redemption_status(
    client: "TwitchHTTPClient",
    params: UpdateRedemptionStatusRequest,
) -> TwitchResponse[RewardRedemption]:
    """Update the status of a custom reward redemption.

    Requires channel:manage:redemptions scope.
    """
    query = {
        "broadcaster_id": params.broadcaster_id,
        "reward_id": params.reward_id,
        "id": params.id,
    }
    data = {"status": params.status}
    response = await client.patch("/channel_points/custom_rewards/redemptions", params=query, data=data)
    return TwitchResponse[RewardRedemption].model_validate(response)



"""Predictions endpoints."""

from typing import TYPE_CHECKING

from twitch_sdk.schemas.base import TwitchResponse
from twitch_sdk.schemas.predictions import (
    CreatePredictionRequest,
    EndPredictionRequest,
    GetPredictionsRequest,
    Prediction,
)

if TYPE_CHECKING:
    from twitch_client import TwitchHTTPClient


async def get_predictions(
    client: "TwitchHTTPClient",
    params: GetPredictionsRequest,
) -> TwitchResponse[Prediction]:
    """Get predictions for a channel.

    Requires channel:read:predictions scope.
    """
    query = params.model_dump(exclude_none=True)
    response = await client.get("/predictions", params=query)
    return TwitchResponse[Prediction].model_validate(response)


async def create_prediction(
    client: "TwitchHTTPClient",
    params: CreatePredictionRequest,
) -> TwitchResponse[Prediction]:
    """Create a prediction on a channel.

    Requires channel:manage:predictions scope.
    """
    data = params.model_dump(exclude_none=True)
    response = await client.post("/predictions", data=data)
    return TwitchResponse[Prediction].model_validate(response)


async def end_prediction(
    client: "TwitchHTTPClient",
    params: EndPredictionRequest,
) -> TwitchResponse[Prediction]:
    """End an active prediction.

    Requires channel:manage:predictions scope.
    """
    data = params.model_dump(exclude_none=True)
    response = await client.patch("/predictions", data=data)
    return TwitchResponse[Prediction].model_validate(response)



"""Pydantic schemas for Hype Train endpoints."""

from datetime import datetime

from pydantic import Field

from .base import TwitchBaseModel


class HypeTrainContribution(TwitchBaseModel):
    """Hype train contribution."""

    total: int
    type: str  # BITS, SUBS, OTHER
    user: str


class HypeTrainEvent(TwitchBaseModel):
    """Hype train event data."""

    id: str
    event_type: str
    event_timestamp: datetime
    version: str
    event_data: dict


class HypeTrain(TwitchBaseModel):
    """Hype train data."""

    id: str
    broadcaster_id: str
    started_at: datetime
    expires_at: datetime
    cooldown_end_time: datetime
    level: int
    goal: int
    total: int
    top_contributions: list[HypeTrainContribution]
    last_contribution: HypeTrainContribution


class GetHypeTrainEventsRequest(TwitchBaseModel):
    """Request params for Get Hype Train Events."""

    broadcaster_id: str
    first: int | None = Field(default=None, le=100)
    after: str | None = None



"""Chat endpoints."""

from typing import TYPE_CHECKING

from twitch_sdk.schemas.base import TwitchResponse
from twitch_sdk.schemas.chat import (
    Badge,
    ChatSettings,
    Chatter,
    Emote,
    GetBadgesRequest,
    GetChatSettingsRequest,
    GetChattersRequest,
    GetChattersResponse,
    GetEmoteSetsRequest,
    GetEmotesRequest,
    GetShoutoutsRequest,
    GetUserChatColorRequest,
    SendAnnouncementRequest,
    SendMessageRequest,
    SendMessageResponse,
    ShoutoutRequest,
    UpdateChatSettingsRequest,
    UpdateUserChatColorRequest,
    UserChatColor,
)

if TYPE_CHECKING:
    from twitch_client import TwitchHTTPClient


async def get_chatters(
    client: "TwitchHTTPClient",
    params: GetChattersRequest,
) -> GetChattersResponse:
    """Get list of users in the broadcaster's chat.

    Requires moderator:read:chatters scope.
    """
    query = params.model_dump(exclude_none=True)
    response = await client.get("/chat/chatters", params=query)
    return GetChattersResponse.model_validate(response)


async def get_channel_emotes(
    client: "TwitchHTTPClient",
    params: GetEmotesRequest,
) -> TwitchResponse[Emote]:
    """Get channel's custom emotes."""
    query = params.model_dump(exclude_none=True)
    response = await client.get("/chat/emotes", params=query)
    return TwitchResponse[Emote].model_validate(response)


async def get_global_emotes(
    client: "TwitchHTTPClient",
) -> TwitchResponse[Emote]:
    """Get global emotes."""
    response = await client.get("/chat/emotes/global")
    return TwitchResponse[Emote].model_validate(response)


async def get_emote_sets(
    client: "TwitchHTTPClient",
    params: GetEmoteSetsRequest,
) -> TwitchResponse[Emote]:
    """Get emotes from specific emote sets."""
    query = params.model_dump(exclude_none=True)
    response = await client.get("/chat/emotes/set", params=query)
    return TwitchResponse[Emote].model_validate(response)


async def get_channel_chat_badges(
    client: "TwitchHTTPClient",
    params: GetBadgesRequest,
) -> TwitchResponse[Badge]:
    """Get channel's chat badges."""
    query = params.model_dump(exclude_none=True)
    response = await client.get("/chat/badges", params=query)
    return TwitchResponse[Badge].model_validate(response)


async def get_global_chat_badges(
    client: "TwitchHTTPClient",
) -> TwitchResponse[Badge]:
    """Get global chat badges."""
    response = await client.get("/chat/badges/global")
    return TwitchResponse[Badge].model_validate(response)


async def get_chat_settings(
    client: "TwitchHTTPClient",
    params: GetChatSettingsRequest,
) -> TwitchResponse[ChatSettings]:
    """Get chat settings for a channel."""
    query = params.model_dump(exclude_none=True)
    response = await client.get("/chat/settings", params=query)
    return TwitchResponse[ChatSettings].model_validate(response)


async def update_chat_settings(
    client: "TwitchHTTPClient",
    params: UpdateChatSettingsRequest,
) -> TwitchResponse[ChatSettings]:
    """Update chat settings for a channel.

    Requires moderator:manage:chat_settings scope.
    """
    query = {"broadcaster_id": params.broadcaster_id, "moderator_id": params.moderator_id}
    data = params.model_dump(exclude={"broadcaster_id", "moderator_id"}, exclude_none=True)
    response = await client.patch("/chat/settings", params=query, data=data)
    return TwitchResponse[ChatSettings].model_validate(response)


async def send_chat_announcement(
    client: "TwitchHTTPClient",
    params: SendAnnouncementRequest,
) -> None:
    """Send an announcement to the chat.

    Requires moderator:manage:announcements scope.
    """
    query = {"broadcaster_id": params.broadcaster_id, "moderator_id": params.moderator_id}
    data = {"message": params.message}
    if params.color:
        data["color"] = params.color
    await client.post("/chat/announcements", params=query, data=data)


async def send_shoutout(
    client: "TwitchHTTPClient",
    params: ShoutoutRequest,
) -> None:
    """Send a shoutout to another broadcaster.

    Requires moderator:manage:shoutouts scope.
    """
    query = params.model_dump(exclude_none=True)
    await client.post("/chat/shoutouts", params=query)


async def get_user_chat_color(
    client: "TwitchHTTPClient",
    params: GetUserChatColorRequest,
) -> TwitchResponse[UserChatColor]:
    """Get the color used for users' names in chat."""
    query = params.model_dump(exclude_none=True)
    response = await client.get("/chat/color", params=query)
    return TwitchResponse[UserChatColor].model_validate(response)


async def update_user_chat_color(
    client: "TwitchHTTPClient",
    params: UpdateUserChatColorRequest,
) -> None:
    """Update the user's chat color.

    Requires user:manage:chat_color scope.
    """
    query = params.model_dump(exclude_none=True)
    await client.put("/chat/color", params=query)


async def send_chat_message(
    client: "TwitchHTTPClient",
    params: SendMessageRequest,
) -> TwitchResponse[SendMessageResponse]:
    """Send a chat message to a broadcaster's chat.

    Requires user:write:chat scope.
    """
    data = params.model_dump(exclude_none=True)
    response = await client.post("/chat/messages", data=data)
    return TwitchResponse[SendMessageResponse].model_validate(response)



"""Pydantic schemas for Charity endpoints."""

from datetime import datetime

from pydantic import Field

from .base import TwitchBaseModel


class CharityAmount(TwitchBaseModel):
    """Charity amount with currency."""

    value: int
    decimal_places: int
    currency: str


class CharityCampaign(TwitchBaseModel):
    """Charity campaign data."""

    id: str
    broadcaster_id: str
    broadcaster_login: str
    broadcaster_name: str
    charity_name: str
    charity_description: str
    charity_logo: str
    charity_website: str
    current_amount: CharityAmount
    target_amount: CharityAmount


class GetCharityCampaignRequest(TwitchBaseModel):
    """Request params for Get Charity Campaign."""

    broadcaster_id: str


class CharityDonation(TwitchBaseModel):
    """Charity donation data."""

    id: str
    campaign_id: str
    user_id: str
    user_login: str
    user_name: str
    amount: CharityAmount


class GetCharityDonationsRequest(TwitchBaseModel):
    """Request params for Get Charity Campaign Donations."""

    broadcaster_id: str
    first: int | None = Field(default=None, le=100)
    after: str | None = None



"""Users endpoints."""

from typing import TYPE_CHECKING

from twitch_sdk.schemas.base import TwitchResponse
from twitch_sdk.schemas.users import (
    BlockUserRequest,
    GetUserBlockListRequest,
    GetUsersRequest,
    UnblockUserRequest,
    UpdateUserExtensionsRequest,
    UpdateUserRequest,
    User,
    UserActiveExtensions,
    UserBlockTarget,
    UserExtension,
)

if TYPE_CHECKING:
    from twitch_client import TwitchHTTPClient


async def get_users(
    client: "TwitchHTTPClient",
    params: GetUsersRequest | None = None,
) -> TwitchResponse[User]:
    """Get information about one or more users.

    If no params provided, returns authenticated user.
    """
    query = params.model_dump(exclude_none=True) if params else {}
    response = await client.get("/users", params=query)
    return TwitchResponse[User].model_validate(response)


async def update_user(
    client: "TwitchHTTPClient",
    params: UpdateUserRequest,
) -> TwitchResponse[User]:
    """Update the authenticated user's description."""
    query = params.model_dump(exclude_none=True)
    response = await client.put("/users", params=query)
    return TwitchResponse[User].model_validate(response)


async def get_user_block_list(
    client: "TwitchHTTPClient",
    params: GetUserBlockListRequest,
) -> TwitchResponse[UserBlockTarget]:
    """Get list of users that the broadcaster has blocked."""
    query = params.model_dump(exclude_none=True)
    response = await client.get("/users/blocks", params=query)
    return TwitchResponse[UserBlockTarget].model_validate(response)


async def block_user(
    client: "TwitchHTTPClient",
    params: BlockUserRequest,
) -> None:
    """Block a user."""
    query = params.model_dump(exclude_none=True)
    await client.put("/users/blocks", params=query)


async def unblock_user(
    client: "TwitchHTTPClient",
    params: UnblockUserRequest,
) -> None:
    """Unblock a user."""
    query = params.model_dump(exclude_none=True)
    await client.delete("/users/blocks", params=query)


async def get_user_extensions(
    client: "TwitchHTTPClient",
) -> TwitchResponse[UserExtension]:
    """Get list of extensions the authenticated user has installed."""
    response = await client.get("/users/extensions/list")
    return TwitchResponse[UserExtension].model_validate(response)


async def get_user_active_extensions(
    client: "TwitchHTTPClient",
    user_id: str | None = None,
) -> dict:
    """Get user's active extensions."""
    params = {"user_id": user_id} if user_id else {}
    response = await client.get("/users/extensions", params=params)
    return response


async def update_user_extensions(
    client: "TwitchHTTPClient",
    params: UpdateUserExtensionsRequest,
) -> dict:
    """Update user's active extensions."""
    response = await client.put("/users/extensions", data=params.model_dump())
    return response



"""Analytics endpoints."""

from typing import TYPE_CHECKING

from twitch_sdk.schemas.base import TwitchResponse
from twitch_sdk.schemas.analytics import (
    ExtensionAnalytics,
    GameAnalytics,
    GetExtensionAnalyticsRequest,
    GetGameAnalyticsRequest,
)

if TYPE_CHECKING:
    from twitch_client import TwitchHTTPClient


async def get_extension_analytics(
    client: "TwitchHTTPClient",
    params: GetExtensionAnalyticsRequest | None = None,
) -> TwitchResponse[ExtensionAnalytics]:
    """Get analytics for extensions.

    Requires analytics:read:extensions scope.
    """
    query = params.model_dump(exclude_none=True) if params else {}
    response = await client.get("/analytics/extensions", params=query)
    return TwitchResponse[ExtensionAnalytics].model_validate(response)


async def get_game_analytics(
    client: "TwitchHTTPClient",
    params: GetGameAnalyticsRequest | None = None,
) -> TwitchResponse[GameAnalytics]:
    """Get analytics for games.

    Requires analytics:read:games scope.
    """
    query = params.model_dump(exclude_none=True) if params else {}
    response = await client.get("/analytics/games", params=query)
    return TwitchResponse[GameAnalytics].model_validate(response)



"""Polls endpoints."""

from typing import TYPE_CHECKING

from twitch_sdk.schemas.base import TwitchResponse
from twitch_sdk.schemas.polls import (
    CreatePollRequest,
    EndPollRequest,
    GetPollsRequest,
    Poll,
)

if TYPE_CHECKING:
    from twitch_client import TwitchHTTPClient


async def get_polls(
    client: "TwitchHTTPClient",
    params: GetPollsRequest,
) -> TwitchResponse[Poll]:
    """Get polls for a channel.

    Requires channel:read:polls scope.
    """
    query = params.model_dump(exclude_none=True)
    response = await client.get("/polls", params=query)
    return TwitchResponse[Poll].model_validate(response)


async def create_poll(
    client: "TwitchHTTPClient",
    params: CreatePollRequest,
) -> TwitchResponse[Poll]:
    """Create a poll on a channel.

    Requires channel:manage:polls scope.
    """
    data = params.model_dump(exclude_none=True)
    response = await client.post("/polls", data=data)
    return TwitchResponse[Poll].model_validate(response)


async def end_poll(
    client: "TwitchHTTPClient",
    params: EndPollRequest,
) -> TwitchResponse[Poll]:
    """End an active poll.

    Requires channel:manage:polls scope.
    """
    data = params.model_dump(exclude_none=True)
    response = await client.patch("/polls", data=data)
    return TwitchResponse[Poll].model_validate(response)



"""Pydantic schemas for Whispers endpoints."""

from pydantic import Field

from .base import TwitchBaseModel


class SendWhisperRequest(TwitchBaseModel):
    """Request to send a whisper."""

    from_user_id: str
    to_user_id: str
    message: str = Field(max_length=10000)



"""Pydantic schemas for Chat endpoints."""

from datetime import datetime

from pydantic import Field

from .base import TwitchBaseModel


class Emote(TwitchBaseModel):
    """Emote data."""

    id: str
    name: str
    images: dict[str, str]
    format: list[str]
    scale: list[str]
    theme_mode: list[str]
    emote_type: str | None = None
    emote_set_id: str | None = None
    owner_id: str | None = None
    tier: str | None = None


class GetEmotesRequest(TwitchBaseModel):
    """Request params for Get Channel Emotes."""

    broadcaster_id: str


class GetEmoteSetsRequest(TwitchBaseModel):
    """Request params for Get Emote Sets."""

    emote_set_id: list[str]


class GetGlobalEmotesRequest(TwitchBaseModel):
    """Request params for Get Global Emotes (no params needed)."""

    pass


class Badge(TwitchBaseModel):
    """Chat badge data."""

    set_id: str
    versions: list[dict]


class GetBadgesRequest(TwitchBaseModel):
    """Request params for Get Channel Chat Badges."""

    broadcaster_id: str


class ChatSettings(TwitchBaseModel):
    """Chat settings data."""

    broadcaster_id: str
    emote_mode: bool
    follower_mode: bool
    follower_mode_duration: int | None = None
    moderator_id: str | None = None
    non_moderator_chat_delay: bool | None = None
    non_moderator_chat_delay_duration: int | None = None
    slow_mode: bool
    slow_mode_wait_time: int | None = None
    subscriber_mode: bool
    unique_chat_mode: bool


class GetChatSettingsRequest(TwitchBaseModel):
    """Request params for Get Chat Settings."""

    broadcaster_id: str
    moderator_id: str | None = None


class UpdateChatSettingsRequest(TwitchBaseModel):
    """Request to update chat settings."""

    broadcaster_id: str
    moderator_id: str
    emote_mode: bool | None = None
    follower_mode: bool | None = None
    follower_mode_duration: int | None = None
    non_moderator_chat_delay: bool | None = None
    non_moderator_chat_delay_duration: int | None = None
    slow_mode: bool | None = None
    slow_mode_wait_time: int | None = None
    subscriber_mode: bool | None = None
    unique_chat_mode: bool | None = None


class Chatter(TwitchBaseModel):
    """Chatter in a channel."""

    user_id: str
    user_login: str
    user_name: str


class GetChattersRequest(TwitchBaseModel):
    """Request params for Get Chatters."""

    broadcaster_id: str
    moderator_id: str
    first: int | None = Field(default=None, le=1000)
    after: str | None = None


class GetChattersResponse(TwitchBaseModel):
    """Response from Get Chatters."""

    data: list[Chatter]
    pagination: dict | None = None
    total: int


class UserChatColor(TwitchBaseModel):
    """User's chat color."""

    user_id: str
    user_login: str
    user_name: str
    color: str


class GetUserChatColorRequest(TwitchBaseModel):
    """Request params for Get User Chat Color."""

    user_id: list[str]


class UpdateUserChatColorRequest(TwitchBaseModel):
    """Request to update chat color."""

    user_id: str
    color: str  # Hex color or named color


class SendMessageRequest(TwitchBaseModel):
    """Request to send a chat message."""

    broadcaster_id: str
    sender_id: str
    message: str
    reply_parent_message_id: str | None = None


class SendMessageResponse(TwitchBaseModel):
    """Response from sending a chat message."""

    message_id: str
    is_sent: bool
    drop_reason: dict | None = None


class Announcement(TwitchBaseModel):
    """Announcement message."""

    message: str
    color: str | None = None  # blue, green, orange, purple, primary


class SendAnnouncementRequest(TwitchBaseModel):
    """Request to send an announcement."""

    broadcaster_id: str
    moderator_id: str
    message: str
    color: str | None = None


class ShoutoutRequest(TwitchBaseModel):
    """Request to send a shoutout."""

    from_broadcaster_id: str
    to_broadcaster_id: str
    moderator_id: str


class GetShoutoutsRequest(TwitchBaseModel):
    """Request params for Get Shoutouts."""

    broadcaster_id: str
    moderator_id: str



"""Pydantic schemas for Teams endpoints."""

from datetime import datetime

from .base import TwitchBaseModel


class TeamMember(TwitchBaseModel):
    """Team member info."""

    user_id: str
    user_login: str
    user_name: str


class Team(TwitchBaseModel):
    """Team data."""

    id: str
    team_name: str
    team_display_name: str
    info: str
    thumbnail_url: str
    banner: str | None = None
    background_image_url: str | None = None
    created_at: datetime
    updated_at: datetime
    users: list[TeamMember] | None = None


class GetTeamsRequest(TwitchBaseModel):
    """Request params for Get Teams."""

    name: str | None = None
    id: str | None = None


class GetChannelTeamsRequest(TwitchBaseModel):
    """Request params for Get Channel Teams."""

    broadcaster_id: str



"""Goals endpoints."""

from typing import TYPE_CHECKING

from twitch_sdk.schemas.base import TwitchResponse
from twitch_sdk.schemas.goals import GetGoalsRequest, Goal

if TYPE_CHECKING:
    from twitch_client import TwitchHTTPClient


async def get_creator_goals(
    client: "TwitchHTTPClient",
    params: GetGoalsRequest,
) -> TwitchResponse[Goal]:
    """Get the broadcaster's active creator goals.

    Requires channel:read:goals scope.
    """
    query = params.model_dump(exclude_none=True)
    response = await client.get("/goals", params=query)
    return TwitchResponse[Goal].model_validate(response)



"""Pydantic schemas for EventSub endpoints."""

from datetime import datetime

from pydantic import Field

from .base import TwitchBaseModel


class EventSubTransport(TwitchBaseModel):
    """EventSub transport configuration."""

    method: str  # webhook, websocket, conduit
    callback: str | None = None
    secret: str | None = None
    session_id: str | None = None
    conduit_id: str | None = None
    connected_at: datetime | None = None
    disconnected_at: datetime | None = None


class EventSubSubscription(TwitchBaseModel):
    """EventSub subscription data."""

    id: str
    status: str  # enabled, webhook_callback_verification_pending, etc.
    type: str
    version: str
    condition: dict
    created_at: datetime
    transport: EventSubTransport
    cost: int


class CreateEventSubSubscriptionRequest(TwitchBaseModel):
    """Request to create an EventSub subscription."""

    type: str
    version: str
    condition: dict
    transport: dict


class GetEventSubSubscriptionsRequest(TwitchBaseModel):
    """Request params for Get EventSub Subscriptions."""

    status: str | None = None
    type: str | None = None
    user_id: str | None = None
    after: str | None = None


class GetEventSubSubscriptionsResponse(TwitchBaseModel):
    """Response from Get EventSub Subscriptions."""

    data: list[EventSubSubscription]
    total: int
    total_cost: int
    max_total_cost: int
    pagination: dict | None = None


class DeleteEventSubSubscriptionRequest(TwitchBaseModel):
    """Request to delete an EventSub subscription."""

    id: str


# WebSocket message types
class WebSocketWelcome(TwitchBaseModel):
    """WebSocket welcome message."""

    id: str
    status: str
    connected_at: datetime
    keepalive_timeout_seconds: int
    reconnect_url: str | None = None

    @property
    def session_id(self) -> str:
        """Alias for id field (Twitch API uses 'id')."""
        return self.id


class WebSocketNotification(TwitchBaseModel):
    """WebSocket notification message."""

    subscription: EventSubSubscription
    event: dict


class WebSocketReconnect(TwitchBaseModel):
    """WebSocket reconnect message."""

    session_id: str
    status: str
    reconnect_url: str


class WebSocketRevocation(TwitchBaseModel):
    """WebSocket subscription revocation message."""

    subscription: EventSubSubscription


# Conduit schemas
class Conduit(TwitchBaseModel):
    """Conduit data."""

    id: str
    shard_count: int


class GetConduitsRequest(TwitchBaseModel):
    """Request params for Get Conduits (no params needed)."""

    pass


class CreateConduitRequest(TwitchBaseModel):
    """Request to create a conduit."""

    shard_count: int = Field(ge=1)


class UpdateConduitRequest(TwitchBaseModel):
    """Request to update a conduit."""

    id: str
    shard_count: int = Field(ge=1)


class DeleteConduitRequest(TwitchBaseModel):
    """Request to delete a conduit."""

    id: str


class ConduitShardTransport(TwitchBaseModel):
    """Conduit shard transport config."""

    method: str
    callback: str | None = None
    session_id: str | None = None


class ConduitShard(TwitchBaseModel):
    """Conduit shard data."""

    id: str
    status: str
    transport: ConduitShardTransport


class GetConduitShardsRequest(TwitchBaseModel):
    """Request params for Get Conduit Shards."""

    conduit_id: str
    status: str | None = None
    after: str | None = None


class UpdateConduitShardsRequest(TwitchBaseModel):
    """Request to update conduit shards."""

    conduit_id: str
    shards: list[dict]



"""Pydantic schemas for Analytics endpoints."""

from datetime import datetime

from pydantic import Field

from .base import DateRange, TwitchBaseModel


class ExtensionAnalytics(TwitchBaseModel):
    """Extension analytics data."""

    extension_id: str
    URL: str
    type: str
    date_range: DateRange


class GetExtensionAnalyticsRequest(TwitchBaseModel):
    """Request params for Get Extension Analytics."""

    extension_id: str | None = None
    type: str | None = None  # overview_v2
    started_at: datetime | None = None
    ended_at: datetime | None = None
    first: int | None = Field(default=None, le=100)
    after: str | None = None


class GameAnalytics(TwitchBaseModel):
    """Game analytics data."""

    game_id: str
    URL: str
    type: str
    date_range: DateRange


class GetGameAnalyticsRequest(TwitchBaseModel):
    """Request params for Get Game Analytics."""

    game_id: str | None = None
    type: str | None = None  # overview_v2
    started_at: datetime | None = None
    ended_at: datetime | None = None
    first: int | None = Field(default=None, le=100)
    after: str | None = None



"""Charity endpoints."""

from typing import TYPE_CHECKING

from twitch_sdk.schemas.base import TwitchResponse
from twitch_sdk.schemas.charity import (
    CharityCampaign,
    CharityDonation,
    GetCharityCampaignRequest,
    GetCharityDonationsRequest,
)

if TYPE_CHECKING:
    from twitch_client import TwitchHTTPClient


async def get_charity_campaign(
    client: "TwitchHTTPClient",
    params: GetCharityCampaignRequest,
) -> TwitchResponse[CharityCampaign]:
    """Get the broadcaster's active charity campaign.

    Requires channel:read:charity scope.
    """
    query = params.model_dump(exclude_none=True)
    response = await client.get("/charity/campaigns", params=query)
    return TwitchResponse[CharityCampaign].model_validate(response)


async def get_charity_campaign_donations(
    client: "TwitchHTTPClient",
    params: GetCharityDonationsRequest,
) -> TwitchResponse[CharityDonation]:
    """Get donations to the broadcaster's charity campaign.

    Requires channel:read:charity scope.
    """
    query = params.model_dump(exclude_none=True)
    response = await client.get("/charity/donations", params=query)
    return TwitchResponse[CharityDonation].model_validate(response)



"""Pydantic schemas for Polls endpoints."""

from datetime import datetime

from pydantic import Field

from .base import TwitchBaseModel


class PollChoice(TwitchBaseModel):
    """Poll choice data."""

    id: str
    title: str
    votes: int = 0
    channel_points_votes: int = 0
    bits_votes: int = 0


class Poll(TwitchBaseModel):
    """Poll data from Twitch API."""

    id: str
    broadcaster_id: str
    broadcaster_login: str
    broadcaster_name: str
    title: str
    choices: list[PollChoice]
    bits_voting_enabled: bool = False
    bits_per_vote: int = 0
    channel_points_voting_enabled: bool = False
    channel_points_per_vote: int = 0
    status: str  # ACTIVE, COMPLETED, TERMINATED, ARCHIVED, MODERATED, INVALID
    duration: int
    started_at: datetime
    ended_at: datetime | None = None


class PollChoiceInput(TwitchBaseModel):
    """Poll choice for creating a poll."""

    title: str = Field(max_length=25)


class CreatePollRequest(TwitchBaseModel):
    """Request to create a poll."""

    broadcaster_id: str
    title: str = Field(max_length=60)
    choices: list[PollChoiceInput] = Field(min_length=2, max_length=5)
    duration: int = Field(ge=15, le=1800)
    channel_points_voting_enabled: bool = False
    channel_points_per_vote: int = Field(default=0, ge=0, le=1000000)


class GetPollsRequest(TwitchBaseModel):
    """Request params for Get Polls."""

    broadcaster_id: str
    id: list[str] | None = None
    first: int | None = Field(default=None, le=20)
    after: str | None = None


class EndPollRequest(TwitchBaseModel):
    """Request to end a poll."""

    broadcaster_id: str
    id: str
    status: str  # TERMINATED or ARCHIVED



"""Pydantic schemas for Predictions endpoints."""

from datetime import datetime

from pydantic import Field

from .base import TwitchBaseModel


class TopPredictor(TwitchBaseModel):
    """Top predictor info."""

    user_id: str
    user_login: str
    user_name: str
    channel_points_used: int
    channel_points_won: int


class PredictionOutcome(TwitchBaseModel):
    """Prediction outcome data."""

    id: str
    title: str
    users: int = 0
    channel_points: int = 0
    top_predictors: list[TopPredictor] | None = None
    color: str  # BLUE, PINK


class Prediction(TwitchBaseModel):
    """Prediction data from Twitch API."""

    id: str
    broadcaster_id: str
    broadcaster_login: str
    broadcaster_name: str
    title: str
    winning_outcome_id: str | None = None
    outcomes: list[PredictionOutcome]
    prediction_window: int
    status: str  # ACTIVE, RESOLVED, CANCELED, LOCKED
    created_at: datetime
    ended_at: datetime | None = None
    locked_at: datetime | None = None


class PredictionOutcomeInput(TwitchBaseModel):
    """Prediction outcome for creating a prediction."""

    title: str = Field(max_length=25)


class CreatePredictionRequest(TwitchBaseModel):
    """Request to create a prediction."""

    broadcaster_id: str
    title: str = Field(max_length=45)
    outcomes: list[PredictionOutcomeInput] = Field(min_length=2, max_length=10)
    prediction_window: int = Field(ge=30, le=1800)


class GetPredictionsRequest(TwitchBaseModel):
    """Request params for Get Predictions."""

    broadcaster_id: str
    id: list[str] | None = None
    first: int | None = Field(default=None, le=25)
    after: str | None = None


class EndPredictionRequest(TwitchBaseModel):
    """Request to end a prediction."""

    broadcaster_id: str
    id: str
    status: str  # RESOLVED, CANCELED, LOCKED
    winning_outcome_id: str | None = None  # Required if RESOLVED



"""Complete Twitch Helix API SDK with Pydantic validation."""

from .client import TwitchSDK
from . import endpoints
from . import schemas

__all__ = [
    "TwitchSDK",
    "endpoints",
    "schemas",
]

__version__ = "0.1.0"



"""Teams endpoints."""

from typing import TYPE_CHECKING

from twitch_sdk.schemas.base import TwitchResponse
from twitch_sdk.schemas.teams import (
    GetChannelTeamsRequest,
    GetTeamsRequest,
    Team,
)

if TYPE_CHECKING:
    from twitch_client import TwitchHTTPClient


async def get_teams(
    client: "TwitchHTTPClient",
    params: GetTeamsRequest,
) -> TwitchResponse[Team]:
    """Get information about a team."""
    query = params.model_dump(exclude_none=True)
    response = await client.get("/teams", params=query)
    return TwitchResponse[Team].model_validate(response)


async def get_channel_teams(
    client: "TwitchHTTPClient",
    params: GetChannelTeamsRequest,
) -> TwitchResponse[Team]:
    """Get teams that a broadcaster is a member of."""
    query = params.model_dump(exclude_none=True)
    response = await client.get("/teams/channel", params=query)
    return TwitchResponse[Team].model_validate(response)



"""Streams endpoints."""

from typing import TYPE_CHECKING

from twitch_sdk.schemas.base import TwitchResponse
from twitch_sdk.schemas.streams import (
    CreateStreamMarkerRequest,
    CreateStreamMarkerResponse,
    GetFollowedStreamsRequest,
    GetStreamKeyRequest,
    GetStreamMarkersRequest,
    GetStreamsRequest,
    Stream,
    StreamKey,
    UserMarkers,
)

if TYPE_CHECKING:
    from twitch_client import TwitchHTTPClient


async def get_streams(
    client: "TwitchHTTPClient",
    params: GetStreamsRequest | None = None,
) -> TwitchResponse[Stream]:
    """Get active streams.

    Returns list of streams matching the query parameters.
    """
    query = params.model_dump(exclude_none=True) if params else {}
    response = await client.get("/streams", params=query)
    return TwitchResponse[Stream].model_validate(response)


async def get_followed_streams(
    client: "TwitchHTTPClient",
    params: GetFollowedStreamsRequest,
) -> TwitchResponse[Stream]:
    """Get streams from channels that the user follows."""
    query = params.model_dump(exclude_none=True)
    response = await client.get("/streams/followed", params=query)
    return TwitchResponse[Stream].model_validate(response)


async def create_stream_marker(
    client: "TwitchHTTPClient",
    params: CreateStreamMarkerRequest,
) -> TwitchResponse[CreateStreamMarkerResponse]:
    """Create a marker in a live stream.

    Requires user:edit:broadcast scope.
    """
    data = params.model_dump(exclude_none=True)
    response = await client.post("/streams/markers", data=data)
    return TwitchResponse[CreateStreamMarkerResponse].model_validate(response)


async def get_stream_markers(
    client: "TwitchHTTPClient",
    params: GetStreamMarkersRequest,
) -> TwitchResponse[UserMarkers]:
    """Get stream markers for a VOD or live stream."""
    query = params.model_dump(exclude_none=True)
    response = await client.get("/streams/markers", params=query)
    return TwitchResponse[UserMarkers].model_validate(response)


async def get_stream_key(
    client: "TwitchHTTPClient",
    params: GetStreamKeyRequest,
) -> TwitchResponse[StreamKey]:
    """Get the channel's stream key.

    Requires channel:read:stream_key scope.
    """
    query = params.model_dump(exclude_none=True)
    response = await client.get("/streams/key", params=query)
    return TwitchResponse[StreamKey].model_validate(response)



"""Channels endpoints."""

from typing import TYPE_CHECKING

from twitch_sdk.schemas.base import TwitchResponse
from twitch_sdk.schemas.channels import (
    AddVIPRequest,
    Channel,
    ChannelEditor,
    FollowedChannel,
    Follower,
    GetChannelEditorsRequest,
    GetChannelFollowersRequest,
    GetChannelFollowersResponse,
    GetChannelInfoRequest,
    GetFollowedChannelsRequest,
    GetVIPsRequest,
    ModifyChannelInfoRequest,
    RemoveVIPRequest,
    VIP,
)

if TYPE_CHECKING:
    from twitch_client import TwitchHTTPClient


async def get_channel_information(
    client: "TwitchHTTPClient",
    params: GetChannelInfoRequest,
) -> TwitchResponse[Channel]:
    """Get channel information for one or more broadcasters."""
    query = params.model_dump(exclude_none=True)
    response = await client.get("/channels", params=query)
    return TwitchResponse[Channel].model_validate(response)


async def modify_channel_information(
    client: "TwitchHTTPClient",
    params: ModifyChannelInfoRequest,
) -> None:
    """Modify channel information.

    Requires channel:manage:broadcast scope.
    """
    query = {"broadcaster_id": params.broadcaster_id}
    data = params.model_dump(exclude={"broadcaster_id"}, exclude_none=True)
    await client.patch("/channels", params=query, data=data)


async def get_channel_editors(
    client: "TwitchHTTPClient",
    params: GetChannelEditorsRequest,
) -> TwitchResponse[ChannelEditor]:
    """Get list of channel editors.

    Requires channel:read:editors scope.
    """
    query = params.model_dump(exclude_none=True)
    response = await client.get("/channels/editors", params=query)
    return TwitchResponse[ChannelEditor].model_validate(response)


async def get_followed_channels(
    client: "TwitchHTTPClient",
    params: GetFollowedChannelsRequest,
) -> TwitchResponse[FollowedChannel]:
    """Get channels that a user follows.

    Requires user:read:follows scope.
    """
    query = params.model_dump(exclude_none=True)
    response = await client.get("/channels/followed", params=query)
    return TwitchResponse[FollowedChannel].model_validate(response)


async def get_channel_followers(
    client: "TwitchHTTPClient",
    params: GetChannelFollowersRequest,
) -> GetChannelFollowersResponse:
    """Get list of users that follow a channel.

    Requires moderator:read:followers scope.
    """
    query = params.model_dump(exclude_none=True)
    response = await client.get("/channels/followers", params=query)
    return GetChannelFollowersResponse.model_validate(response)


async def get_vips(
    client: "TwitchHTTPClient",
    params: GetVIPsRequest,
) -> TwitchResponse[VIP]:
    """Get list of VIPs for a channel.

    Requires channel:read:vips or channel:manage:vips scope.
    """
    query = params.model_dump(exclude_none=True)
    response = await client.get("/channels/vips", params=query)
    return TwitchResponse[VIP].model_validate(response)


async def add_channel_vip(
    client: "TwitchHTTPClient",
    params: AddVIPRequest,
) -> None:
    """Add a VIP to the channel.

    Requires channel:manage:vips scope.
    """
    query = params.model_dump(exclude_none=True)
    await client.post("/channels/vips", params=query)


async def remove_channel_vip(
    client: "TwitchHTTPClient",
    params: RemoveVIPRequest,
) -> None:
    """Remove a VIP from the channel.

    Requires channel:manage:vips scope.
    """
    query = params.model_dump(exclude_none=True)
    await client.delete("/channels/vips", params=query)



"""Games endpoints."""

from typing import TYPE_CHECKING

from twitch_sdk.schemas.base import TwitchResponse
from twitch_sdk.schemas.games import (
    Game,
    GetGamesRequest,
    GetTopGamesRequest,
)

if TYPE_CHECKING:
    from twitch_client import TwitchHTTPClient


async def get_games(
    client: "TwitchHTTPClient",
    params: GetGamesRequest,
) -> TwitchResponse[Game]:
    """Get games by ID, name, or IGDB ID."""
    query = params.model_dump(exclude_none=True)
    response = await client.get("/games", params=query)
    return TwitchResponse[Game].model_validate(response)


async def get_top_games(
    client: "TwitchHTTPClient",
    params: GetTopGamesRequest | None = None,
) -> TwitchResponse[Game]:
    """Get top games by viewer count."""
    query = params.model_dump(exclude_none=True) if params else {}
    response = await client.get("/games/top", params=query)
    return TwitchResponse[Game].model_validate(response)



"""Main Twitch SDK client."""

from twitch_client import TwitchAuth, TwitchCredentials, TwitchHTTPClient

from . import endpoints
from .endpoints.eventsub import EventSubWebSocket


class TwitchSDK:
    """Main Twitch SDK client.

    Provides access to all Twitch Helix API endpoints through
    a unified interface with Pydantic validation.
    """

    def __init__(
        self,
        credentials: TwitchCredentials | None = None,
        http_client: TwitchHTTPClient | None = None,
    ):
        """Initialize the SDK.

        Args:
            credentials: Twitch credentials. If None, loads from environment.
            http_client: Pre-configured HTTP client. If None, creates one.
        """
        if http_client:
            self._client = http_client
            self._owns_client = False
        else:
            self._client = TwitchHTTPClient(credentials=credentials)
            self._owns_client = True

    @property
    def http(self) -> TwitchHTTPClient:
        """Get the underlying HTTP client."""
        return self._client

    # Endpoint access
    @property
    def ads(self):
        """Access Ads endpoints."""
        return endpoints.ads

    @property
    def analytics(self):
        """Access Analytics endpoints."""
        return endpoints.analytics

    @property
    def bits(self):
        """Access Bits endpoints."""
        return endpoints.bits

    @property
    def channel_points(self):
        """Access Channel Points endpoints."""
        return endpoints.channel_points

    @property
    def channels(self):
        """Access Channels endpoints."""
        return endpoints.channels

    @property
    def charity(self):
        """Access Charity endpoints."""
        return endpoints.charity

    @property
    def chat(self):
        """Access Chat endpoints."""
        return endpoints.chat

    @property
    def clips(self):
        """Access Clips endpoints."""
        return endpoints.clips

    @property
    def eventsub(self):
        """Access EventSub endpoints."""
        return endpoints.eventsub

    @property
    def games(self):
        """Access Games endpoints."""
        return endpoints.games

    @property
    def goals(self):
        """Access Goals endpoints."""
        return endpoints.goals

    @property
    def guest_star(self):
        """Access Guest Star endpoints."""
        return endpoints.guest_star

    @property
    def hype_train(self):
        """Access Hype Train endpoints."""
        return endpoints.hype_train

    @property
    def moderation(self):
        """Access Moderation endpoints."""
        return endpoints.moderation

    @property
    def polls(self):
        """Access Polls endpoints."""
        return endpoints.polls

    @property
    def predictions(self):
        """Access Predictions endpoints."""
        return endpoints.predictions

    @property
    def raids(self):
        """Access Raids endpoints."""
        return endpoints.raids

    @property
    def schedule(self):
        """Access Schedule endpoints."""
        return endpoints.schedule

    @property
    def search(self):
        """Access Search endpoints."""
        return endpoints.search

    @property
    def streams(self):
        """Access Streams endpoints."""
        return endpoints.streams

    @property
    def subscriptions(self):
        """Access Subscriptions endpoints."""
        return endpoints.subscriptions

    @property
    def teams(self):
        """Access Teams endpoints."""
        return endpoints.teams

    @property
    def users(self):
        """Access Users endpoints."""
        return endpoints.users

    @property
    def videos(self):
        """Access Videos endpoints."""
        return endpoints.videos

    @property
    def whispers(self):
        """Access Whispers endpoints."""
        return endpoints.whispers

    def create_eventsub_websocket(self) -> EventSubWebSocket:
        """Create an EventSub WebSocket client.

        Returns:
            EventSubWebSocket instance for subscribing to real-time events.
        """
        return EventSubWebSocket(self._client)

    async def close(self) -> None:
        """Close the SDK and release resources."""
        if self._owns_client:
            await self._client.close()

    async def __aenter__(self) -> "TwitchSDK":
        """Async context manager entry."""
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb) -> None:
        """Async context manager exit."""
        await self.close()



"""Pydantic schemas for Search endpoints."""

from datetime import datetime

from pydantic import Field

from .base import TwitchBaseModel


class SearchChannel(TwitchBaseModel):
    """Channel from search results."""

    broadcaster_language: str
    broadcaster_login: str
    display_name: str
    game_id: str
    game_name: str
    id: str
    is_live: bool
    tags: list[str]
    thumbnail_url: str
    title: str
    started_at: datetime | None = None


class SearchChannelsRequest(TwitchBaseModel):
    """Request params for Search Channels."""

    query: str
    live_only: bool = False
    first: int | None = Field(default=None, le=100)
    after: str | None = None


class SearchCategory(TwitchBaseModel):
    """Category from search results."""

    id: str
    name: str
    box_art_url: str


class SearchCategoriesRequest(TwitchBaseModel):
    """Request params for Search Categories."""

    query: str
    first: int | None = Field(default=None, le=100)
    after: str | None = None



"""Pydantic schemas for Moderation endpoints."""

from datetime import datetime

from pydantic import Field

from .base import TwitchBaseModel


class BannedUser(TwitchBaseModel):
    """Banned user data."""

    user_id: str
    user_login: str
    user_name: str
    expires_at: datetime | None = None
    created_at: datetime
    reason: str
    moderator_id: str
    moderator_login: str
    moderator_name: str


class GetBannedUsersRequest(TwitchBaseModel):
    """Request params for Get Banned Users."""

    broadcaster_id: str
    user_id: list[str] | None = None
    first: int | None = Field(default=None, le=100)
    before: str | None = None
    after: str | None = None


class BanUserRequest(TwitchBaseModel):
    """Request to ban a user."""

    broadcaster_id: str
    moderator_id: str
    data: "BanUserData"


class BanUserData(TwitchBaseModel):
    """Ban user data payload."""

    user_id: str
    duration: int | None = None  # Seconds, None for permanent
    reason: str | None = None


class BanUserResponse(TwitchBaseModel):
    """Response from banning a user."""

    broadcaster_id: str
    moderator_id: str
    user_id: str
    created_at: datetime
    end_time: datetime | None = None


class UnbanUserRequest(TwitchBaseModel):
    """Request to unban a user."""

    broadcaster_id: str
    moderator_id: str
    user_id: str


class UnbanRequest(TwitchBaseModel):
    """Unban request data."""

    id: str
    broadcaster_id: str
    broadcaster_login: str
    broadcaster_name: str
    moderator_id: str
    moderator_login: str
    moderator_name: str
    user_id: str
    user_login: str
    user_name: str
    text: str
    status: str
    created_at: datetime
    resolved_at: datetime | None = None
    resolution_text: str | None = None


class GetUnbanRequestsRequest(TwitchBaseModel):
    """Request params for Get Unban Requests."""

    broadcaster_id: str
    moderator_id: str
    status: str | None = None  # pending, approved, denied, acknowledged, canceled
    user_id: str | None = None
    first: int | None = Field(default=None, le=100)
    after: str | None = None


class ResolveUnbanRequestRequest(TwitchBaseModel):
    """Request to resolve an unban request."""

    broadcaster_id: str
    moderator_id: str
    unban_request_id: str
    status: str  # approved, denied
    resolution_text: str | None = None


class BlockedTerm(TwitchBaseModel):
    """Blocked term data."""

    broadcaster_id: str
    moderator_id: str
    id: str
    text: str
    created_at: datetime
    updated_at: datetime
    expires_at: datetime | None = None


class GetBlockedTermsRequest(TwitchBaseModel):
    """Request params for Get Blocked Terms."""

    broadcaster_id: str
    moderator_id: str
    first: int | None = Field(default=None, le=100)
    after: str | None = None


class AddBlockedTermRequest(TwitchBaseModel):
    """Request to add a blocked term."""

    broadcaster_id: str
    moderator_id: str
    text: str = Field(min_length=2, max_length=500)


class RemoveBlockedTermRequest(TwitchBaseModel):
    """Request to remove a blocked term."""

    broadcaster_id: str
    moderator_id: str
    id: str


class AutoModSettings(TwitchBaseModel):
    """AutoMod settings data."""

    broadcaster_id: str
    moderator_id: str
    overall_level: int | None = None
    disability: int
    aggression: int
    sexuality_sex_or_gender: int
    misogyny: int
    bullying: int
    swearing: int
    race_ethnicity_or_religion: int
    sex_based_terms: int


class GetAutoModSettingsRequest(TwitchBaseModel):
    """Request params for Get AutoMod Settings."""

    broadcaster_id: str
    moderator_id: str


class UpdateAutoModSettingsRequest(TwitchBaseModel):
    """Request to update AutoMod settings."""

    broadcaster_id: str
    moderator_id: str
    overall_level: int | None = Field(default=None, ge=0, le=4)
    aggression: int | None = Field(default=None, ge=0, le=4)
    bullying: int | None = Field(default=None, ge=0, le=4)
    disability: int | None = Field(default=None, ge=0, le=4)
    misogyny: int | None = Field(default=None, ge=0, le=4)
    race_ethnicity_or_religion: int | None = Field(default=None, ge=0, le=4)
    sex_based_terms: int | None = Field(default=None, ge=0, le=4)
    sexuality_sex_or_gender: int | None = Field(default=None, ge=0, le=4)
    swearing: int | None = Field(default=None, ge=0, le=4)


class Moderator(TwitchBaseModel):
    """Moderator data."""

    user_id: str
    user_login: str
    user_name: str


class GetModeratorsRequest(TwitchBaseModel):
    """Request params for Get Moderators."""

    broadcaster_id: str
    user_id: list[str] | None = None
    first: int | None = Field(default=None, le=100)
    after: str | None = None


class AddModeratorRequest(TwitchBaseModel):
    """Request to add a moderator."""

    broadcaster_id: str
    user_id: str


class RemoveModeratorRequest(TwitchBaseModel):
    """Request to remove a moderator."""

    broadcaster_id: str
    user_id: str


class ShieldModeStatus(TwitchBaseModel):
    """Shield mode status."""

    is_active: bool
    moderator_id: str
    moderator_login: str
    moderator_name: str
    last_activated_at: datetime


class GetShieldModeStatusRequest(TwitchBaseModel):
    """Request params for Get Shield Mode Status."""

    broadcaster_id: str
    moderator_id: str


class UpdateShieldModeStatusRequest(TwitchBaseModel):
    """Request to update shield mode status."""

    broadcaster_id: str
    moderator_id: str
    is_active: bool


class WarnUserRequest(TwitchBaseModel):
    """Request to warn a user."""

    broadcaster_id: str
    moderator_id: str
    user_id: str
    reason: str = Field(max_length=500)


class DeleteChatMessagesRequest(TwitchBaseModel):
    """Request to delete chat messages."""

    broadcaster_id: str
    moderator_id: str
    message_id: str | None = None  # If None, clears all messages


class ManageHeldAutoModMessageRequest(TwitchBaseModel):
    """Request to manage a held AutoMod message."""

    user_id: str
    msg_id: str
    action: str  # ALLOW, DENY



"""Pydantic schemas for Schedule endpoints."""

from datetime import datetime

from pydantic import Field

from .base import TwitchBaseModel


class ScheduleCategory(TwitchBaseModel):
    """Category for a scheduled segment."""

    id: str
    name: str


class ScheduleSegment(TwitchBaseModel):
    """Scheduled stream segment."""

    id: str
    start_time: datetime
    end_time: datetime
    title: str
    canceled_until: datetime | None = None
    category: ScheduleCategory | None = None
    is_recurring: bool


class ScheduleVacation(TwitchBaseModel):
    """Vacation period in schedule."""

    start_time: datetime
    end_time: datetime


class Schedule(TwitchBaseModel):
    """Channel schedule."""

    segments: list[ScheduleSegment] | None = None
    broadcaster_id: str
    broadcaster_login: str
    broadcaster_name: str
    vacation: ScheduleVacation | None = None


class GetScheduleRequest(TwitchBaseModel):
    """Request params for Get Channel Stream Schedule."""

    broadcaster_id: str
    id: list[str] | None = None
    start_time: datetime | None = None
    utc_offset: str | None = None
    first: int | None = Field(default=None, le=25)
    after: str | None = None


class GetScheduleICalendarRequest(TwitchBaseModel):
    """Request params for Get Channel iCalendar."""

    broadcaster_id: str


class UpdateScheduleRequest(TwitchBaseModel):
    """Request to update channel schedule settings."""

    broadcaster_id: str
    is_vacation_enabled: bool | None = None
    vacation_start_time: datetime | None = None
    vacation_end_time: datetime | None = None
    timezone: str | None = None


class CreateScheduleSegmentRequest(TwitchBaseModel):
    """Request to create a schedule segment."""

    broadcaster_id: str
    start_time: datetime
    timezone: str
    duration: int = Field(ge=30, le=1440)  # minutes
    is_recurring: bool = False
    category_id: str | None = None
    title: str | None = Field(default=None, max_length=140)


class UpdateScheduleSegmentRequest(TwitchBaseModel):
    """Request to update a schedule segment."""

    broadcaster_id: str
    id: str
    start_time: datetime | None = None
    timezone: str | None = None
    duration: int | None = Field(default=None, ge=30, le=1440)
    is_canceled: bool | None = None
    category_id: str | None = None
    title: str | None = Field(default=None, max_length=140)


class DeleteScheduleSegmentRequest(TwitchBaseModel):
    """Request to delete a schedule segment."""

    broadcaster_id: str
    id: str



"""Pydantic schemas for Videos endpoints."""

from datetime import datetime

from pydantic import Field

from .base import TwitchBaseModel


class MutedSegment(TwitchBaseModel):
    """Muted segment in a video."""

    duration: int
    offset: int


class Video(TwitchBaseModel):
    """Video data from Twitch API."""

    id: str
    stream_id: str | None = None
    user_id: str
    user_login: str
    user_name: str
    title: str
    description: str
    created_at: datetime
    published_at: datetime
    url: str
    thumbnail_url: str
    viewable: str
    view_count: int
    language: str
    type: str  # archive, highlight, upload
    duration: str
    muted_segments: list[MutedSegment] | None = None


class GetVideosRequest(TwitchBaseModel):
    """Request params for Get Videos."""

    id: list[str] | None = None
    user_id: str | None = None
    game_id: str | None = None
    language: str | None = None
    period: str | None = None  # all, day, week, month
    sort: str | None = None  # time, trending, views
    type: str | None = None  # all, archive, highlight, upload
    first: int | None = Field(default=None, le=100)
    before: str | None = None
    after: str | None = None


class DeleteVideosRequest(TwitchBaseModel):
    """Request to delete videos."""

    id: list[str] = Field(max_length=5)



"""Guest Star endpoints."""

from typing import TYPE_CHECKING

from twitch_sdk.schemas.base import TwitchResponse
from twitch_sdk.schemas.guest_star import (
    AssignGuestStarSlotRequest,
    CreateGuestStarSessionRequest,
    DeleteGuestStarInviteRequest,
    DeleteGuestStarSlotRequest,
    EndGuestStarSessionRequest,
    GetGuestStarInvitesRequest,
    GetGuestStarSessionRequest,
    GetGuestStarSettingsRequest,
    GuestStarInvite,
    GuestStarSession,
    GuestStarSettings,
    SendGuestStarInviteRequest,
    UpdateGuestStarSettingsRequest,
    UpdateGuestStarSlotRequest,
    UpdateGuestStarSlotSettingsRequest,
)

if TYPE_CHECKING:
    from twitch_client import TwitchHTTPClient


async def get_channel_guest_star_settings(
    client: "TwitchHTTPClient",
    params: GetGuestStarSettingsRequest,
) -> TwitchResponse[GuestStarSettings]:
    """Get guest star settings for a channel.

    Requires channel:read:guest_star or channel:manage:guest_star scope.
    """
    query = params.model_dump(exclude_none=True)
    response = await client.get("/guest_star/channel_settings", params=query)
    return TwitchResponse[GuestStarSettings].model_validate(response)


async def update_channel_guest_star_settings(
    client: "TwitchHTTPClient",
    params: UpdateGuestStarSettingsRequest,
) -> None:
    """Update guest star settings for a channel.

    Requires channel:manage:guest_star scope.
    """
    query = {"broadcaster_id": params.broadcaster_id}
    data = params.model_dump(exclude={"broadcaster_id"}, exclude_none=True)
    await client.put("/guest_star/channel_settings", params=query, data=data)


async def get_guest_star_session(
    client: "TwitchHTTPClient",
    params: GetGuestStarSessionRequest,
) -> TwitchResponse[GuestStarSession]:
    """Get active guest star session for a channel.

    Requires channel:read:guest_star or channel:manage:guest_star scope.
    """
    query = params.model_dump(exclude_none=True)
    response = await client.get("/guest_star/session", params=query)
    return TwitchResponse[GuestStarSession].model_validate(response)


async def create_guest_star_session(
    client: "TwitchHTTPClient",
    params: CreateGuestStarSessionRequest,
) -> TwitchResponse[GuestStarSession]:
    """Create a guest star session.

    Requires channel:manage:guest_star scope.
    """
    query = params.model_dump(exclude_none=True)
    response = await client.post("/guest_star/session", params=query)
    return TwitchResponse[GuestStarSession].model_validate(response)


async def end_guest_star_session(
    client: "TwitchHTTPClient",
    params: EndGuestStarSessionRequest,
) -> TwitchResponse[GuestStarSession]:
    """End a guest star session.

    Requires channel:manage:guest_star scope.
    """
    query = params.model_dump(exclude_none=True)
    response = await client.delete("/guest_star/session", params=query)
    return TwitchResponse[GuestStarSession].model_validate(response)


async def get_guest_star_invites(
    client: "TwitchHTTPClient",
    params: GetGuestStarInvitesRequest,
) -> TwitchResponse[GuestStarInvite]:
    """Get guest star invites for a session.

    Requires channel:read:guest_star or channel:manage:guest_star scope.
    """
    query = params.model_dump(exclude_none=True)
    response = await client.get("/guest_star/invites", params=query)
    return TwitchResponse[GuestStarInvite].model_validate(response)


async def send_guest_star_invite(
    client: "TwitchHTTPClient",
    params: SendGuestStarInviteRequest,
) -> None:
    """Send a guest star invite.

    Requires channel:manage:guest_star scope.
    """
    query = params.model_dump(exclude_none=True)
    await client.post("/guest_star/invites", params=query)


async def delete_guest_star_invite(
    client: "TwitchHTTPClient",
    params: DeleteGuestStarInviteRequest,
) -> None:
    """Delete a guest star invite.

    Requires channel:manage:guest_star scope.
    """
    query = params.model_dump(exclude_none=True)
    await client.delete("/guest_star/invites", params=query)


async def assign_guest_star_slot(
    client: "TwitchHTTPClient",
    params: AssignGuestStarSlotRequest,
) -> None:
    """Assign a guest to a slot.

    Requires channel:manage:guest_star scope.
    """
    query = params.model_dump(exclude_none=True)
    await client.post("/guest_star/slot", params=query)


async def update_guest_star_slot(
    client: "TwitchHTTPClient",
    params: UpdateGuestStarSlotRequest,
) -> None:
    """Move a guest between slots.

    Requires channel:manage:guest_star scope.
    """
    query = params.model_dump(exclude_none=True)
    await client.patch("/guest_star/slot", params=query)


async def delete_guest_star_slot(
    client: "TwitchHTTPClient",
    params: DeleteGuestStarSlotRequest,
) -> None:
    """Remove a guest from a slot.

    Requires channel:manage:guest_star scope.
    """
    query = params.model_dump(exclude_none=True)
    await client.delete("/guest_star/slot", params=query)


async def update_guest_star_slot_settings(
    client: "TwitchHTTPClient",
    params: UpdateGuestStarSlotSettingsRequest,
) -> None:
    """Update slot settings (audio/video/volume).

    Requires channel:manage:guest_star scope.
    """
    query = {
        "broadcaster_id": params.broadcaster_id,
        "moderator_id": params.moderator_id,
        "session_id": params.session_id,
        "slot_id": params.slot_id,
    }
    data = params.model_dump(
        exclude={"broadcaster_id", "moderator_id", "session_id", "slot_id"},
        exclude_none=True,
    )
    await client.patch("/guest_star/slot_settings", params=query, data=data)



"""Pydantic schemas for Channels endpoints."""

from datetime import datetime

from pydantic import Field

from .base import TwitchBaseModel


class Channel(TwitchBaseModel):
    """Channel information."""

    broadcaster_id: str
    broadcaster_login: str
    broadcaster_name: str
    broadcaster_language: str
    game_id: str
    game_name: str
    title: str
    delay: int
    tags: list[str] = Field(default_factory=list)
    content_classification_labels: list[str] = Field(default_factory=list)
    is_branded_content: bool


class GetChannelInfoRequest(TwitchBaseModel):
    """Request params for Get Channel Information."""

    broadcaster_id: list[str]


class ModifyChannelInfoRequest(TwitchBaseModel):
    """Request to modify channel information."""

    broadcaster_id: str
    game_id: str | None = None
    broadcaster_language: str | None = None
    title: str | None = None
    delay: int | None = None
    tags: list[str] | None = None
    content_classification_labels: list[dict] | None = None
    is_branded_content: bool | None = None


class ChannelEditor(TwitchBaseModel):
    """Channel editor info."""

    user_id: str
    user_name: str
    created_at: datetime


class GetChannelEditorsRequest(TwitchBaseModel):
    """Request params for Get Channel Editors."""

    broadcaster_id: str


class FollowedChannel(TwitchBaseModel):
    """Channel that a user follows."""

    broadcaster_id: str
    broadcaster_login: str
    broadcaster_name: str
    followed_at: datetime


class GetFollowedChannelsRequest(TwitchBaseModel):
    """Request params for Get Followed Channels."""

    user_id: str
    broadcaster_id: str | None = None
    first: int | None = Field(default=None, le=100)
    after: str | None = None


class Follower(TwitchBaseModel):
    """Channel follower info."""

    user_id: str
    user_login: str
    user_name: str
    followed_at: datetime


class GetChannelFollowersRequest(TwitchBaseModel):
    """Request params for Get Channel Followers."""

    broadcaster_id: str
    user_id: str | None = None
    first: int | None = Field(default=None, le=100)
    after: str | None = None


class GetChannelFollowersResponse(TwitchBaseModel):
    """Response from Get Channel Followers."""

    data: list[Follower]
    pagination: dict | None = None
    total: int


class VIP(TwitchBaseModel):
    """VIP user info."""

    user_id: str
    user_login: str
    user_name: str


class GetVIPsRequest(TwitchBaseModel):
    """Request params for Get VIPs."""

    broadcaster_id: str
    user_id: list[str] | None = None
    first: int | None = Field(default=None, le=100)
    after: str | None = None


class AddVIPRequest(TwitchBaseModel):
    """Request to add a VIP."""

    broadcaster_id: str
    user_id: str


class RemoveVIPRequest(TwitchBaseModel):
    """Request to remove a VIP."""

    broadcaster_id: str
    user_id: str



"""Moderation endpoints."""

from typing import TYPE_CHECKING

from twitch_sdk.schemas.base import TwitchResponse
from twitch_sdk.schemas.moderation import (
    AddBlockedTermRequest,
    AddModeratorRequest,
    AutoModSettings,
    BannedUser,
    BanUserRequest,
    BanUserResponse,
    BlockedTerm,
    DeleteChatMessagesRequest,
    GetAutoModSettingsRequest,
    GetBannedUsersRequest,
    GetBlockedTermsRequest,
    GetModeratorsRequest,
    GetShieldModeStatusRequest,
    GetUnbanRequestsRequest,
    ManageHeldAutoModMessageRequest,
    Moderator,
    RemoveBlockedTermRequest,
    RemoveModeratorRequest,
    ResolveUnbanRequestRequest,
    ShieldModeStatus,
    UnbanRequest,
    UnbanUserRequest,
    UpdateAutoModSettingsRequest,
    UpdateShieldModeStatusRequest,
    WarnUserRequest,
)

if TYPE_CHECKING:
    from twitch_client import TwitchHTTPClient


async def get_banned_users(
    client: "TwitchHTTPClient",
    params: GetBannedUsersRequest,
) -> TwitchResponse[BannedUser]:
    """Get list of banned users for a channel.

    Requires moderation:read scope.
    """
    query = params.model_dump(exclude_none=True)
    response = await client.get("/moderation/banned", params=query)
    return TwitchResponse[BannedUser].model_validate(response)


async def ban_user(
    client: "TwitchHTTPClient",
    params: BanUserRequest,
) -> TwitchResponse[BanUserResponse]:
    """Ban a user from a channel.

    Requires moderator:manage:banned_users scope.
    """
    query = {"broadcaster_id": params.broadcaster_id, "moderator_id": params.moderator_id}
    data = {"data": params.data.model_dump(exclude_none=True)}
    response = await client.post("/moderation/bans", params=query, data=data)
    return TwitchResponse[BanUserResponse].model_validate(response)


async def unban_user(
    client: "TwitchHTTPClient",
    params: UnbanUserRequest,
) -> None:
    """Unban a user from a channel.

    Requires moderator:manage:banned_users scope.
    """
    query = params.model_dump(exclude_none=True)
    await client.delete("/moderation/bans", params=query)


async def get_unban_requests(
    client: "TwitchHTTPClient",
    params: GetUnbanRequestsRequest,
) -> TwitchResponse[UnbanRequest]:
    """Get unban requests for a channel.

    Requires moderator:read:unban_requests scope.
    """
    query = params.model_dump(exclude_none=True)
    response = await client.get("/moderation/unban_requests", params=query)
    return TwitchResponse[UnbanRequest].model_validate(response)


async def resolve_unban_request(
    client: "TwitchHTTPClient",
    params: ResolveUnbanRequestRequest,
) -> TwitchResponse[UnbanRequest]:
    """Resolve an unban request.

    Requires moderator:manage:unban_requests scope.
    """
    query = params.model_dump(exclude_none=True)
    response = await client.patch("/moderation/unban_requests", params=query)
    return TwitchResponse[UnbanRequest].model_validate(response)


async def get_blocked_terms(
    client: "TwitchHTTPClient",
    params: GetBlockedTermsRequest,
) -> TwitchResponse[BlockedTerm]:
    """Get blocked terms for a channel.

    Requires moderator:read:blocked_terms scope.
    """
    query = params.model_dump(exclude_none=True)
    response = await client.get("/moderation/blocked_terms", params=query)
    return TwitchResponse[BlockedTerm].model_validate(response)


async def add_blocked_term(
    client: "TwitchHTTPClient",
    params: AddBlockedTermRequest,
) -> TwitchResponse[BlockedTerm]:
    """Add a blocked term to a channel.

    Requires moderator:manage:blocked_terms scope.
    """
    query = {"broadcaster_id": params.broadcaster_id, "moderator_id": params.moderator_id}
    data = {"text": params.text}
    response = await client.post("/moderation/blocked_terms", params=query, data=data)
    return TwitchResponse[BlockedTerm].model_validate(response)


async def remove_blocked_term(
    client: "TwitchHTTPClient",
    params: RemoveBlockedTermRequest,
) -> None:
    """Remove a blocked term from a channel.

    Requires moderator:manage:blocked_terms scope.
    """
    query = params.model_dump(exclude_none=True)
    await client.delete("/moderation/blocked_terms", params=query)


async def get_automod_settings(
    client: "TwitchHTTPClient",
    params: GetAutoModSettingsRequest,
) -> TwitchResponse[AutoModSettings]:
    """Get AutoMod settings for a channel.

    Requires moderator:read:automod_settings scope.
    """
    query = params.model_dump(exclude_none=True)
    response = await client.get("/moderation/automod/settings", params=query)
    return TwitchResponse[AutoModSettings].model_validate(response)


async def update_automod_settings(
    client: "TwitchHTTPClient",
    params: UpdateAutoModSettingsRequest,
) -> TwitchResponse[AutoModSettings]:
    """Update AutoMod settings for a channel.

    Requires moderator:manage:automod_settings scope.
    """
    query = {"broadcaster_id": params.broadcaster_id, "moderator_id": params.moderator_id}
    data = params.model_dump(exclude={"broadcaster_id", "moderator_id"}, exclude_none=True)
    response = await client.put("/moderation/automod/settings", params=query, data=data)
    return TwitchResponse[AutoModSettings].model_validate(response)


async def manage_held_automod_message(
    client: "TwitchHTTPClient",
    params: ManageHeldAutoModMessageRequest,
) -> None:
    """Allow or deny a message held by AutoMod.

    Requires moderator:manage:automod scope.
    """
    data = params.model_dump(exclude_none=True)
    await client.post("/moderation/automod/message", data=data)


async def get_moderators(
    client: "TwitchHTTPClient",
    params: GetModeratorsRequest,
) -> TwitchResponse[Moderator]:
    """Get list of moderators for a channel.

    Requires moderation:read scope.
    """
    query = params.model_dump(exclude_none=True)
    response = await client.get("/moderation/moderators", params=query)
    return TwitchResponse[Moderator].model_validate(response)


async def add_moderator(
    client: "TwitchHTTPClient",
    params: AddModeratorRequest,
) -> None:
    """Add a moderator to a channel.

    Requires channel:manage:moderators scope.
    """
    query = params.model_dump(exclude_none=True)
    await client.post("/moderation/moderators", params=query)


async def remove_moderator(
    client: "TwitchHTTPClient",
    params: RemoveModeratorRequest,
) -> None:
    """Remove a moderator from a channel.

    Requires channel:manage:moderators scope.
    """
    query = params.model_dump(exclude_none=True)
    await client.delete("/moderation/moderators", params=query)


async def get_shield_mode_status(
    client: "TwitchHTTPClient",
    params: GetShieldModeStatusRequest,
) -> TwitchResponse[ShieldModeStatus]:
    """Get shield mode status for a channel.

    Requires moderator:read:shield_mode scope.
    """
    query = params.model_dump(exclude_none=True)
    response = await client.get("/moderation/shield_mode", params=query)
    return TwitchResponse[ShieldModeStatus].model_validate(response)


async def update_shield_mode_status(
    client: "TwitchHTTPClient",
    params: UpdateShieldModeStatusRequest,
) -> TwitchResponse[ShieldModeStatus]:
    """Update shield mode status for a channel.

    Requires moderator:manage:shield_mode scope.
    """
    query = {"broadcaster_id": params.broadcaster_id, "moderator_id": params.moderator_id}
    data = {"is_active": params.is_active}
    response = await client.put("/moderation/shield_mode", params=query, data=data)
    return TwitchResponse[ShieldModeStatus].model_validate(response)


async def warn_chat_user(
    client: "TwitchHTTPClient",
    params: WarnUserRequest,
) -> None:
    """Warn a user in chat.

    Requires moderator:manage:warnings scope.
    """
    query = {"broadcaster_id": params.broadcaster_id, "moderator_id": params.moderator_id}
    data = {"user_id": params.user_id, "reason": params.reason}
    await client.post("/moderation/warnings", params=query, data=data)


async def delete_chat_messages(
    client: "TwitchHTTPClient",
    params: DeleteChatMessagesRequest,
) -> None:
    """Delete chat messages.

    Requires moderator:manage:chat_messages scope.
    """
    query = params.model_dump(exclude_none=True)
    await client.delete("/moderation/chat", params=query)



"""Pydantic schemas for Goals endpoints."""

from datetime import datetime

from .base import TwitchBaseModel


class Goal(TwitchBaseModel):
    """Creator goal data."""

    id: str
    broadcaster_id: str
    broadcaster_login: str
    broadcaster_name: str
    type: str  # follower, subscription, subscription_count, new_subscription, new_subscription_count
    description: str
    current_amount: int
    target_amount: int
    created_at: datetime


class GetGoalsRequest(TwitchBaseModel):
    """Request params for Get Creator Goals."""

    broadcaster_id: str



"""Subscriptions endpoints."""

from typing import TYPE_CHECKING

from twitch_sdk.schemas.base import TwitchResponse
from twitch_sdk.schemas.subscriptions import (
    CheckUserSubscriptionRequest,
    GetBroadcasterSubscriptionsRequest,
    GetBroadcasterSubscriptionsResponse,
    UserSubscription,
)

if TYPE_CHECKING:
    from twitch_client import TwitchHTTPClient


async def get_broadcaster_subscriptions(
    client: "TwitchHTTPClient",
    params: GetBroadcasterSubscriptionsRequest,
) -> GetBroadcasterSubscriptionsResponse:
    """Get list of subscribers for a broadcaster.

    Requires channel:read:subscriptions scope.
    """
    query = params.model_dump(exclude_none=True)
    response = await client.get("/subscriptions", params=query)
    return GetBroadcasterSubscriptionsResponse.model_validate(response)


async def check_user_subscription(
    client: "TwitchHTTPClient",
    params: CheckUserSubscriptionRequest,
) -> TwitchResponse[UserSubscription]:
    """Check if a user is subscribed to a broadcaster.

    Requires user:read:subscriptions scope.
    """
    query = params.model_dump(exclude_none=True)
    response = await client.get("/subscriptions/user", params=query)
    return TwitchResponse[UserSubscription].model_validate(response)



"""Schedule endpoints."""

from typing import TYPE_CHECKING

from twitch_sdk.schemas.base import TwitchResponse
from twitch_sdk.schemas.schedule import (
    CreateScheduleSegmentRequest,
    DeleteScheduleSegmentRequest,
    GetScheduleICalendarRequest,
    GetScheduleRequest,
    Schedule,
    ScheduleSegment,
    UpdateScheduleRequest,
    UpdateScheduleSegmentRequest,
)

if TYPE_CHECKING:
    from twitch_client import TwitchHTTPClient


async def get_channel_stream_schedule(
    client: "TwitchHTTPClient",
    params: GetScheduleRequest,
) -> dict:
    """Get a broadcaster's streaming schedule."""
    query = params.model_dump(exclude_none=True)
    response = await client.get("/schedule", params=query)
    return response


async def get_channel_icalendar(
    client: "TwitchHTTPClient",
    params: GetScheduleICalendarRequest,
) -> str:
    """Get a broadcaster's schedule as iCalendar data."""
    query = params.model_dump(exclude_none=True)
    response = await client.get("/schedule/icalendar", params=query)
    return response


async def update_channel_stream_schedule(
    client: "TwitchHTTPClient",
    params: UpdateScheduleRequest,
) -> None:
    """Update a broadcaster's streaming schedule settings.

    Requires channel:manage:schedule scope.
    """
    query = params.model_dump(exclude_none=True)
    await client.patch("/schedule/settings", params=query)


async def create_channel_stream_schedule_segment(
    client: "TwitchHTTPClient",
    params: CreateScheduleSegmentRequest,
) -> dict:
    """Create a scheduled broadcast segment.

    Requires channel:manage:schedule scope.
    """
    query = {"broadcaster_id": params.broadcaster_id}
    data = params.model_dump(exclude={"broadcaster_id"}, exclude_none=True)
    response = await client.post("/schedule/segment", params=query, data=data)
    return response


async def update_channel_stream_schedule_segment(
    client: "TwitchHTTPClient",
    params: UpdateScheduleSegmentRequest,
) -> dict:
    """Update a scheduled broadcast segment.

    Requires channel:manage:schedule scope.
    """
    query = {"broadcaster_id": params.broadcaster_id, "id": params.id}
    data = params.model_dump(exclude={"broadcaster_id", "id"}, exclude_none=True)
    response = await client.patch("/schedule/segment", params=query, data=data)
    return response


async def delete_channel_stream_schedule_segment(
    client: "TwitchHTTPClient",
    params: DeleteScheduleSegmentRequest,
) -> None:
    """Delete a scheduled broadcast segment.

    Requires channel:manage:schedule scope.
    """
    query = params.model_dump(exclude_none=True)
    await client.delete("/schedule/segment", params=query)



"""EventSub endpoints and WebSocket handler."""

import asyncio
import json
from typing import TYPE_CHECKING, AsyncGenerator

import websockets

from twitch_sdk.schemas.base import TwitchResponse
from twitch_sdk.schemas.eventsub import (
    Conduit,
    ConduitShard,
    CreateConduitRequest,
    CreateEventSubSubscriptionRequest,
    DeleteConduitRequest,
    DeleteEventSubSubscriptionRequest,
    EventSubSubscription,
    GetConduitShardsRequest,
    GetEventSubSubscriptionsRequest,
    GetEventSubSubscriptionsResponse,
    UpdateConduitRequest,
    UpdateConduitShardsRequest,
    WebSocketNotification,
    WebSocketWelcome,
)

if TYPE_CHECKING:
    from twitch_client import TwitchHTTPClient


# HTTP API Endpoints

async def create_eventsub_subscription(
    client: "TwitchHTTPClient",
    params: CreateEventSubSubscriptionRequest,
) -> TwitchResponse[EventSubSubscription]:
    """Create an EventSub subscription."""
    data = params.model_dump(exclude_none=True)
    response = await client.post("/eventsub/subscriptions", data=data)
    return TwitchResponse[EventSubSubscription].model_validate(response)


async def delete_eventsub_subscription(
    client: "TwitchHTTPClient",
    params: DeleteEventSubSubscriptionRequest,
) -> None:
    """Delete an EventSub subscription."""
    query = params.model_dump(exclude_none=True)
    await client.delete("/eventsub/subscriptions", params=query)


async def get_eventsub_subscriptions(
    client: "TwitchHTTPClient",
    params: GetEventSubSubscriptionsRequest | None = None,
) -> GetEventSubSubscriptionsResponse:
    """Get list of EventSub subscriptions."""
    query = params.model_dump(exclude_none=True) if params else {}
    response = await client.get("/eventsub/subscriptions", params=query)
    return GetEventSubSubscriptionsResponse.model_validate(response)


# Conduit endpoints

async def get_conduits(
    client: "TwitchHTTPClient",
) -> TwitchResponse[Conduit]:
    """Get list of conduits."""
    response = await client.get("/eventsub/conduits")
    return TwitchResponse[Conduit].model_validate(response)


async def create_conduit(
    client: "TwitchHTTPClient",
    params: CreateConduitRequest,
) -> TwitchResponse[Conduit]:
    """Create a conduit."""
    data = params.model_dump(exclude_none=True)
    response = await client.post("/eventsub/conduits", data=data)
    return TwitchResponse[Conduit].model_validate(response)


async def update_conduit(
    client: "TwitchHTTPClient",
    params: UpdateConduitRequest,
) -> TwitchResponse[Conduit]:
    """Update a conduit's shard count."""
    data = params.model_dump(exclude_none=True)
    response = await client.patch("/eventsub/conduits", data=data)
    return TwitchResponse[Conduit].model_validate(response)


async def delete_conduit(
    client: "TwitchHTTPClient",
    params: DeleteConduitRequest,
) -> None:
    """Delete a conduit."""
    query = params.model_dump(exclude_none=True)
    await client.delete("/eventsub/conduits", params=query)


async def get_conduit_shards(
    client: "TwitchHTTPClient",
    params: GetConduitShardsRequest,
) -> TwitchResponse[ConduitShard]:
    """Get shards for a conduit."""
    query = params.model_dump(exclude_none=True)
    response = await client.get("/eventsub/conduits/shards", params=query)
    return TwitchResponse[ConduitShard].model_validate(response)


async def update_conduit_shards(
    client: "TwitchHTTPClient",
    params: UpdateConduitShardsRequest,
) -> TwitchResponse[ConduitShard]:
    """Update conduit shards."""
    data = params.model_dump(exclude_none=True)
    response = await client.patch("/eventsub/conduits/shards", data=data)
    return TwitchResponse[ConduitShard].model_validate(response)


# WebSocket client for EventSub

class EventSubWebSocket:
    """WebSocket client for Twitch EventSub.

    Connects to Twitch EventSub WebSocket, handles session management,
    and yields incoming events.
    """

    EVENTSUB_WSS_URL = "wss://eventsub.wss.twitch.tv/ws"

    def __init__(self, client: "TwitchHTTPClient"):
        """Initialize EventSub WebSocket.

        Args:
            client: TwitchHTTPClient for making API calls.
        """
        self.client = client
        self._ws = None
        self._session_id: str | None = None
        self._keepalive_timeout: int = 10
        self._reconnect_url: str | None = None
        self._subscriptions: list[str] = []

    @property
    def session_id(self) -> str | None:
        """Get the current session ID."""
        return self._session_id

    @property
    def is_connected(self) -> bool:
        """Check if WebSocket is connected."""
        return self._ws is not None and self._ws.open

    async def connect(self) -> str:
        """Connect to EventSub WebSocket.

        Returns:
            Session ID for creating subscriptions.
        """
        url = self._reconnect_url or self.EVENTSUB_WSS_URL
        self._ws = await websockets.connect(url)

        # Wait for welcome message
        raw_message = await self._ws.recv()
        message = json.loads(raw_message)

        if message.get("metadata", {}).get("message_type") != "session_welcome":
            raise RuntimeError(f"Expected session_welcome, got: {message}")

        payload = message.get("payload", {}).get("session", {})
        welcome = WebSocketWelcome.model_validate(payload)

        self._session_id = welcome.session_id
        self._keepalive_timeout = welcome.keepalive_timeout_seconds
        self._reconnect_url = welcome.reconnect_url

        return self._session_id

    async def subscribe(
        self,
        event_type: str,
        version: str,
        condition: dict,
    ) -> EventSubSubscription:
        """Subscribe to an EventSub event type.

        Args:
            event_type: The event type (e.g., "channel.chat.message").
            version: The subscription version.
            condition: The condition for the subscription.

        Returns:
            The created subscription.
        """
        if not self._session_id:
            raise RuntimeError("Not connected. Call connect() first.")

        params = CreateEventSubSubscriptionRequest(
            type=event_type,
            version=version,
            condition=condition,
            transport={
                "method": "websocket",
                "session_id": self._session_id,
            },
        )

        result = await create_eventsub_subscription(self.client, params)
        subscription = result.data[0]
        self._subscriptions.append(subscription.id)
        return subscription

    async def events(self) -> AsyncGenerator[dict, None]:
        """Async generator that yields EventSub events.

        Handles keepalive messages internally.
        Yields event payloads for notification messages.
        """
        if not self._ws:
            raise RuntimeError("Not connected. Call connect() first.")

        while True:
            try:
                raw_message = await asyncio.wait_for(
                    self._ws.recv(),
                    timeout=self._keepalive_timeout + 10,
                )
            except asyncio.TimeoutError:
                # No message received, connection may be dead
                break

            message = json.loads(raw_message)
            message_type = message.get("metadata", {}).get("message_type")

            if message_type == "session_keepalive":
                # Keepalive, continue
                continue

            elif message_type == "notification":
                # Yield the event
                payload = message.get("payload", {})
                yield payload

            elif message_type == "session_reconnect":
                # Handle reconnection
                payload = message.get("payload", {}).get("session", {})
                self._reconnect_url = payload.get("reconnect_url")
                await self._ws.close()
                await self.connect()

            elif message_type == "revocation":
                # Subscription was revoked
                payload = message.get("payload", {})
                yield {"revocation": payload}

    async def close(self) -> None:
        """Close the WebSocket connection."""
        if self._ws:
            await self._ws.close()
            self._ws = None
            self._session_id = None

    async def __aenter__(self) -> "EventSubWebSocket":
        """Async context manager entry."""
        await self.connect()
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb) -> None:
        """Async context manager exit."""
        await self.close()



"""Pydantic schemas for Guest Star endpoints."""

from datetime import datetime

from pydantic import Field

from .base import TwitchBaseModel


class GuestStarSettings(TwitchBaseModel):
    """Guest star session settings."""

    is_moderator_send_live_enabled: bool
    slot_count: int
    is_browser_source_audio_enabled: bool
    group_layout: str
    browser_source_token: str | None = None


class GetGuestStarSettingsRequest(TwitchBaseModel):
    """Request params for Get Guest Star Channel Settings."""

    broadcaster_id: str
    moderator_id: str


class UpdateGuestStarSettingsRequest(TwitchBaseModel):
    """Request to update guest star settings."""

    broadcaster_id: str
    is_moderator_send_live_enabled: bool | None = None
    slot_count: int | None = Field(default=None, ge=1, le=6)
    is_browser_source_audio_enabled: bool | None = None
    group_layout: str | None = None  # TILED_LAYOUT, SCREENSHARE_LAYOUT, HORIZONTAL_LAYOUT, VERTICAL_LAYOUT


class GuestStarGuest(TwitchBaseModel):
    """Guest in a guest star session."""

    slot_id: str
    is_live: bool
    user_id: str
    user_login: str
    user_display_name: str
    volume: int
    assigned_at: datetime
    audio_settings: dict
    video_settings: dict


class GuestStarSession(TwitchBaseModel):
    """Guest star session data."""

    id: str
    guests: list[GuestStarGuest]


class GetGuestStarSessionRequest(TwitchBaseModel):
    """Request params for Get Guest Star Session."""

    broadcaster_id: str
    moderator_id: str


class CreateGuestStarSessionRequest(TwitchBaseModel):
    """Request to create a guest star session."""

    broadcaster_id: str


class EndGuestStarSessionRequest(TwitchBaseModel):
    """Request to end a guest star session."""

    broadcaster_id: str
    session_id: str


class GuestStarInvite(TwitchBaseModel):
    """Guest star invite."""

    user_id: str
    invited_at: datetime
    status: str
    is_audio_enabled: bool
    is_video_enabled: bool
    is_audio_available: bool
    is_video_available: bool


class GetGuestStarInvitesRequest(TwitchBaseModel):
    """Request params for Get Guest Star Invites."""

    broadcaster_id: str
    moderator_id: str
    session_id: str


class SendGuestStarInviteRequest(TwitchBaseModel):
    """Request to send a guest star invite."""

    broadcaster_id: str
    moderator_id: str
    session_id: str
    guest_id: str


class DeleteGuestStarInviteRequest(TwitchBaseModel):
    """Request to delete a guest star invite."""

    broadcaster_id: str
    moderator_id: str
    session_id: str
    guest_id: str


class AssignGuestStarSlotRequest(TwitchBaseModel):
    """Request to assign a guest to a slot."""

    broadcaster_id: str
    moderator_id: str
    session_id: str
    guest_id: str
    slot_id: str


class UpdateGuestStarSlotRequest(TwitchBaseModel):
    """Request to update a guest star slot."""

    broadcaster_id: str
    moderator_id: str
    session_id: str
    source_slot_id: str
    destination_slot_id: str | None = None


class DeleteGuestStarSlotRequest(TwitchBaseModel):
    """Request to remove a guest from a slot."""

    broadcaster_id: str
    moderator_id: str
    session_id: str
    guest_id: str
    slot_id: str
    should_reinvite_guest: bool | None = None


class UpdateGuestStarSlotSettingsRequest(TwitchBaseModel):
    """Request to update slot settings."""

    broadcaster_id: str
    moderator_id: str
    session_id: str
    slot_id: str
    is_audio_enabled: bool | None = None
    is_video_enabled: bool | None = None
    is_live: bool | None = None
    volume: int | None = Field(default=None, ge=0, le=100)



"""Base schemas used across endpoints."""

from datetime import datetime
from typing import Generic, TypeVar

from pydantic import BaseModel, ConfigDict

T = TypeVar("T")


class TwitchBaseModel(BaseModel):
    """Base model for all Twitch schemas."""

    model_config = ConfigDict(
        populate_by_name=True,
        extra="ignore",  # Ignore unknown fields from API
    )


class Pagination(TwitchBaseModel):
    """Pagination info in API responses."""

    cursor: str | None = None


class TwitchResponse(TwitchBaseModel, Generic[T]):
    """Generic wrapper for Twitch API responses with data array."""

    data: list[T]
    pagination: Pagination | None = None
    total: int | None = None


class DateRange(TwitchBaseModel):
    """Date range for analytics."""

    started_at: datetime
    ended_at: datetime



"""Pydantic schemas for Channel Points endpoints."""

from datetime import datetime

from pydantic import Field

from .base import TwitchBaseModel


class CustomRewardImage(TwitchBaseModel):
    """Custom reward image URLs."""

    url_1x: str
    url_2x: str
    url_4x: str


class MaxPerStreamSetting(TwitchBaseModel):
    """Max per stream setting."""

    is_enabled: bool
    max_per_stream: int


class MaxPerUserPerStreamSetting(TwitchBaseModel):
    """Max per user per stream setting."""

    is_enabled: bool
    max_per_user_per_stream: int


class GlobalCooldownSetting(TwitchBaseModel):
    """Global cooldown setting."""

    is_enabled: bool
    global_cooldown_seconds: int


class CustomReward(TwitchBaseModel):
    """Custom channel points reward."""

    broadcaster_id: str
    broadcaster_login: str
    broadcaster_name: str
    id: str
    title: str
    prompt: str
    cost: int
    image: CustomRewardImage | None = None
    default_image: CustomRewardImage
    background_color: str
    is_enabled: bool
    is_user_input_required: bool
    max_per_stream_setting: MaxPerStreamSetting
    max_per_user_per_stream_setting: MaxPerUserPerStreamSetting
    global_cooldown_setting: GlobalCooldownSetting
    is_paused: bool
    is_in_stock: bool
    should_redemptions_skip_request_queue: bool
    redemptions_redeemed_current_stream: int | None = None
    cooldown_expires_at: datetime | None = None


class GetCustomRewardsRequest(TwitchBaseModel):
    """Request params for Get Custom Reward."""

    broadcaster_id: str
    id: list[str] | None = None
    only_manageable_rewards: bool = False


class CreateCustomRewardRequest(TwitchBaseModel):
    """Request to create a custom reward."""

    broadcaster_id: str
    title: str = Field(max_length=45)
    cost: int = Field(ge=1)
    prompt: str | None = Field(default=None, max_length=200)
    is_enabled: bool = True
    background_color: str | None = None
    is_user_input_required: bool = False
    is_max_per_stream_enabled: bool = False
    max_per_stream: int | None = None
    is_max_per_user_per_stream_enabled: bool = False
    max_per_user_per_stream: int | None = None
    is_global_cooldown_enabled: bool = False
    global_cooldown_seconds: int | None = None
    should_redemptions_skip_request_queue: bool = False


class UpdateCustomRewardRequest(TwitchBaseModel):
    """Request to update a custom reward."""

    broadcaster_id: str
    id: str
    title: str | None = Field(default=None, max_length=45)
    cost: int | None = Field(default=None, ge=1)
    prompt: str | None = Field(default=None, max_length=200)
    is_enabled: bool | None = None
    background_color: str | None = None
    is_user_input_required: bool | None = None
    is_max_per_stream_enabled: bool | None = None
    max_per_stream: int | None = None
    is_max_per_user_per_stream_enabled: bool | None = None
    max_per_user_per_stream: int | None = None
    is_global_cooldown_enabled: bool | None = None
    global_cooldown_seconds: int | None = None
    should_redemptions_skip_request_queue: bool | None = None
    is_paused: bool | None = None


class DeleteCustomRewardRequest(TwitchBaseModel):
    """Request to delete a custom reward."""

    broadcaster_id: str
    id: str


class RewardRedemption(TwitchBaseModel):
    """Channel points reward redemption."""

    broadcaster_id: str
    broadcaster_login: str
    broadcaster_name: str
    id: str
    user_id: str
    user_login: str
    user_name: str
    user_input: str
    status: str  # CANCELED, FULFILLED, UNFULFILLED
    redeemed_at: datetime
    reward: dict


class GetCustomRewardRedemptionRequest(TwitchBaseModel):
    """Request params for Get Custom Reward Redemption."""

    broadcaster_id: str
    reward_id: str
    id: list[str] | None = None
    status: str | None = None  # CANCELED, FULFILLED, UNFULFILLED
    sort: str | None = None  # OLDEST, NEWEST
    first: int | None = Field(default=None, le=50)
    after: str | None = None


class UpdateRedemptionStatusRequest(TwitchBaseModel):
    """Request to update redemption status."""

    broadcaster_id: str
    reward_id: str
    id: list[str]
    status: str  # CANCELED, FULFILLED



"""Pydantic schemas for Games endpoints."""

from pydantic import Field

from .base import TwitchBaseModel


class Game(TwitchBaseModel):
    """Game/category data."""

    id: str
    name: str
    box_art_url: str
    igdb_id: str | None = None


class GetGamesRequest(TwitchBaseModel):
    """Request params for Get Games."""

    id: list[str] | None = None
    name: list[str] | None = None
    igdb_id: list[str] | None = None


class GetTopGamesRequest(TwitchBaseModel):
    """Request params for Get Top Games."""

    first: int | None = Field(default=None, le=100)
    before: str | None = None
    after: str | None = None



"""Pydantic schemas for Subscriptions endpoints."""

from pydantic import Field

from .base import TwitchBaseModel


class Subscription(TwitchBaseModel):
    """Subscription data."""

    broadcaster_id: str
    broadcaster_login: str
    broadcaster_name: str
    gifter_id: str | None = None
    gifter_login: str | None = None
    gifter_name: str | None = None
    is_gift: bool
    plan_name: str
    tier: str  # 1000, 2000, 3000
    user_id: str
    user_login: str
    user_name: str


class GetBroadcasterSubscriptionsRequest(TwitchBaseModel):
    """Request params for Get Broadcaster Subscriptions."""

    broadcaster_id: str
    user_id: list[str] | None = None
    first: int | None = Field(default=None, le=100)
    after: str | None = None


class GetBroadcasterSubscriptionsResponse(TwitchBaseModel):
    """Response from Get Broadcaster Subscriptions."""

    data: list[Subscription]
    pagination: dict | None = None
    total: int
    points: int


class UserSubscription(TwitchBaseModel):
    """User's subscription to a broadcaster."""

    broadcaster_id: str
    broadcaster_login: str
    broadcaster_name: str
    gifter_id: str | None = None
    gifter_login: str | None = None
    gifter_name: str | None = None
    is_gift: bool
    tier: str


class CheckUserSubscriptionRequest(TwitchBaseModel):
    """Request params for Check User Subscription."""

    broadcaster_id: str
    user_id: str



"""Whispers endpoints."""

from typing import TYPE_CHECKING

from twitch_sdk.schemas.whispers import SendWhisperRequest

if TYPE_CHECKING:
    from twitch_client import TwitchHTTPClient


async def send_whisper(
    client: "TwitchHTTPClient",
    params: SendWhisperRequest,
) -> None:
    """Send a whisper to another user.

    Requires user:manage:whispers scope.
    """
    query = {"from_user_id": params.from_user_id, "to_user_id": params.to_user_id}
    data = {"message": params.message}
    await client.post("/whispers", params=query, data=data)



"""Pydantic schemas for Twitch Helix API."""

from .base import DateRange, Pagination, TwitchBaseModel, TwitchResponse

# Re-export all schemas for convenience
from . import (
    ads,
    analytics,
    bits,
    channel_points,
    channels,
    charity,
    chat,
    clips,
    eventsub,
    games,
    goals,
    guest_star,
    hype_train,
    moderation,
    polls,
    predictions,
    raids,
    schedule,
    search,
    streams,
    subscriptions,
    teams,
    users,
    videos,
    whispers,
)

__all__ = [
    "TwitchBaseModel",
    "TwitchResponse",
    "Pagination",
    "DateRange",
    "ads",
    "analytics",
    "bits",
    "channel_points",
    "channels",
    "charity",
    "chat",
    "clips",
    "eventsub",
    "games",
    "goals",
    "guest_star",
    "hype_train",
    "moderation",
    "polls",
    "predictions",
    "raids",
    "schedule",
    "search",
    "streams",
    "subscriptions",
    "teams",
    "users",
    "videos",
    "whispers",
]



"""Search endpoints."""

from typing import TYPE_CHECKING

from twitch_sdk.schemas.base import TwitchResponse
from twitch_sdk.schemas.search import (
    SearchCategoriesRequest,
    SearchCategory,
    SearchChannel,
    SearchChannelsRequest,
)

if TYPE_CHECKING:
    from twitch_client import TwitchHTTPClient


async def search_categories(
    client: "TwitchHTTPClient",
    params: SearchCategoriesRequest,
) -> TwitchResponse[SearchCategory]:
    """Search for categories/games."""
    query = params.model_dump(exclude_none=True)
    response = await client.get("/search/categories", params=query)
    return TwitchResponse[SearchCategory].model_validate(response)


async def search_channels(
    client: "TwitchHTTPClient",
    params: SearchChannelsRequest,
) -> TwitchResponse[SearchChannel]:
    """Search for channels."""
    query = params.model_dump(exclude_none=True)
    response = await client.get("/search/channels", params=query)
    return TwitchResponse[SearchChannel].model_validate(response)



"""Pydantic schemas for Ads endpoints."""

from datetime import datetime

from pydantic import Field

from .base import TwitchBaseModel


class StartCommercialRequest(TwitchBaseModel):
    """Request to start a commercial."""

    broadcaster_id: str
    length: int  # 30, 60, 90, 120, 150, 180


class StartCommercialResponse(TwitchBaseModel):
    """Response from starting a commercial."""

    length: int
    message: str
    retry_after: int


class AdSchedule(TwitchBaseModel):
    """Ad schedule information."""

    snooze_count: int
    snooze_refresh_at: datetime | None = None
    next_ad_at: datetime | None = None
    duration: int
    last_ad_at: datetime | None = None
    preroll_free_time: int


class GetAdScheduleRequest(TwitchBaseModel):
    """Request params for Get Ad Schedule."""

    broadcaster_id: str


class SnoozeNextAdRequest(TwitchBaseModel):
    """Request to snooze the next ad."""

    broadcaster_id: str


class SnoozeNextAdResponse(TwitchBaseModel):
    """Response from snoozing an ad."""

    snooze_count: int
    snooze_refresh_at: datetime | None = None
    next_ad_at: datetime | None = None



"""Pydantic schemas for Clips endpoints."""

from datetime import datetime

from pydantic import Field

from .base import TwitchBaseModel


class Clip(TwitchBaseModel):
    """Clip data from Twitch API."""

    id: str
    url: str
    embed_url: str
    broadcaster_id: str
    broadcaster_name: str
    creator_id: str
    creator_name: str
    video_id: str
    game_id: str
    language: str
    title: str
    view_count: int
    created_at: datetime
    thumbnail_url: str
    duration: float
    vod_offset: int | None = None
    is_featured: bool


class CreateClipRequest(TwitchBaseModel):
    """Request to create a clip."""

    broadcaster_id: str
    has_delay: bool = False


class CreateClipResponse(TwitchBaseModel):
    """Response from creating a clip."""

    id: str
    edit_url: str


class GetClipsRequest(TwitchBaseModel):
    """Request params for Get Clips."""

    broadcaster_id: str | None = None
    game_id: str | None = None
    id: list[str] | None = None
    started_at: datetime | None = None
    ended_at: datetime | None = None
    first: int | None = Field(default=None, le=100)
    before: str | None = None
    after: str | None = None
    is_featured: bool | None = None



"""Clips endpoints."""

from typing import TYPE_CHECKING

from twitch_sdk.schemas.base import TwitchResponse
from twitch_sdk.schemas.clips import (
    Clip,
    CreateClipRequest,
    CreateClipResponse,
    GetClipsRequest,
)

if TYPE_CHECKING:
    from twitch_client import TwitchHTTPClient


async def create_clip(
    client: "TwitchHTTPClient",
    params: CreateClipRequest,
) -> TwitchResponse[CreateClipResponse]:
    """Create a clip from the broadcaster's stream.

    Requires clips:edit scope.
    """
    query = params.model_dump(exclude_none=True)
    response = await client.post("/clips", params=query)
    return TwitchResponse[CreateClipResponse].model_validate(response)


async def get_clips(
    client: "TwitchHTTPClient",
    params: GetClipsRequest,
) -> TwitchResponse[Clip]:
    """Get clips for a broadcaster, game, or specific clip IDs."""
    query = params.model_dump(exclude_none=True)
    response = await client.get("/clips", params=query)
    return TwitchResponse[Clip].model_validate(response)



"""Pydantic schemas for Bits endpoints."""

from datetime import datetime

from pydantic import Field

from .base import DateRange, TwitchBaseModel


class BitsLeaderboardEntry(TwitchBaseModel):
    """Entry in the bits leaderboard."""

    user_id: str
    user_login: str
    user_name: str
    rank: int
    score: int


class GetBitsLeaderboardRequest(TwitchBaseModel):
    """Request params for Get Bits Leaderboard."""

    count: int | None = Field(default=None, le=100)
    period: str | None = None  # day, week, month, year, all
    started_at: datetime | None = None
    user_id: str | None = None


class GetBitsLeaderboardResponse(TwitchBaseModel):
    """Response from Get Bits Leaderboard."""

    data: list[BitsLeaderboardEntry]
    date_range: DateRange
    total: int


class CheermoteImage(TwitchBaseModel):
    """Cheermote image URLs."""

    url_1x: str
    url_2x: str
    url_4x: str


class CheermoteTier(TwitchBaseModel):
    """Cheermote tier data."""

    min_bits: int
    id: str
    color: str
    images: dict[str, dict[str, CheermoteImage]]
    can_cheer: bool
    show_in_bits_card: bool


class Cheermote(TwitchBaseModel):
    """Cheermote data."""

    prefix: str
    tiers: list[CheermoteTier]
    type: str
    order: int
    last_updated: datetime
    is_charitable: bool


class GetCheermotesRequest(TwitchBaseModel):
    """Request params for Get Cheermotes."""

    broadcaster_id: str | None = None


class ExtensionTransaction(TwitchBaseModel):
    """Extension transaction data."""

    id: str
    timestamp: datetime
    broadcaster_id: str
    broadcaster_login: str
    broadcaster_name: str
    user_id: str
    user_login: str
    user_name: str
    product_type: str
    product_data: dict


class GetExtensionTransactionsRequest(TwitchBaseModel):
    """Request params for Get Extension Transactions."""

    extension_id: str
    id: list[str] | None = None
    first: int | None = Field(default=None, le=100)
    after: str | None = None



"""Pydantic schemas for Streams endpoints."""

from datetime import datetime

from pydantic import Field

from .base import TwitchBaseModel


class Stream(TwitchBaseModel):
    """Stream data from Twitch API."""

    id: str
    user_id: str
    user_login: str
    user_name: str
    game_id: str
    game_name: str
    type: str  # "live" or ""
    title: str
    tags: list[str] = Field(default_factory=list)
    viewer_count: int
    started_at: datetime
    language: str
    thumbnail_url: str
    is_mature: bool


class GetStreamsRequest(TwitchBaseModel):
    """Request params for Get Streams endpoint."""

    user_id: list[str] | None = None
    user_login: list[str] | None = None
    game_id: list[str] | None = None
    type: str | None = None  # all, live
    language: list[str] | None = None
    first: int | None = Field(default=None, le=100)
    before: str | None = None
    after: str | None = None


class GetFollowedStreamsRequest(TwitchBaseModel):
    """Request params for Get Followed Streams."""

    user_id: str
    first: int | None = Field(default=None, le=100)
    after: str | None = None


class StreamMarker(TwitchBaseModel):
    """Stream marker data."""

    id: str
    created_at: datetime
    position_seconds: int
    description: str


class CreateStreamMarkerRequest(TwitchBaseModel):
    """Request to create a stream marker."""

    user_id: str
    description: str | None = Field(default=None, max_length=140)


class CreateStreamMarkerResponse(TwitchBaseModel):
    """Response from creating a stream marker."""

    id: str
    created_at: datetime
    position_seconds: int
    description: str


class GetStreamMarkersRequest(TwitchBaseModel):
    """Request params for Get Stream Markers."""

    user_id: str | None = None
    video_id: str | None = None
    first: int | None = Field(default=None, le=100)
    before: str | None = None
    after: str | None = None


class VideoMarkers(TwitchBaseModel):
    """Markers for a video."""

    video_id: str
    markers: list[StreamMarker]


class UserMarkers(TwitchBaseModel):
    """Markers for a user's videos."""

    user_id: str
    user_login: str
    user_name: str
    videos: list[VideoMarkers]


class StreamKey(TwitchBaseModel):
    """Stream key data."""

    stream_key: str


class GetStreamKeyRequest(TwitchBaseModel):
    """Request params for Get Stream Key."""

    broadcaster_id: str



"""Videos endpoints."""

from typing import TYPE_CHECKING

from twitch_sdk.schemas.base import TwitchResponse
from twitch_sdk.schemas.videos import (
    DeleteVideosRequest,
    GetVideosRequest,
    Video,
)

if TYPE_CHECKING:
    from twitch_client import TwitchHTTPClient


async def get_videos(
    client: "TwitchHTTPClient",
    params: GetVideosRequest,
) -> TwitchResponse[Video]:
    """Get videos by ID, user, or game."""
    query = params.model_dump(exclude_none=True)
    response = await client.get("/videos", params=query)
    return TwitchResponse[Video].model_validate(response)


async def delete_videos(
    client: "TwitchHTTPClient",
    params: DeleteVideosRequest,
) -> list[str]:
    """Delete videos.

    Requires channel:manage:videos scope.
    Returns list of deleted video IDs.
    """
    query = params.model_dump(exclude_none=True)
    response = await client.delete("/videos", params=query)
    return response.get("data", [])



"""Bits endpoints."""

from typing import TYPE_CHECKING

from twitch_sdk.schemas.base import TwitchResponse
from twitch_sdk.schemas.bits import (
    Cheermote,
    ExtensionTransaction,
    GetBitsLeaderboardRequest,
    GetBitsLeaderboardResponse,
    GetCheermotesRequest,
    GetExtensionTransactionsRequest,
)

if TYPE_CHECKING:
    from twitch_client import TwitchHTTPClient


async def get_bits_leaderboard(
    client: "TwitchHTTPClient",
    params: GetBitsLeaderboardRequest | None = None,
) -> GetBitsLeaderboardResponse:
    """Get bits leaderboard for a channel.

    Requires bits:read scope.
    """
    query = params.model_dump(exclude_none=True) if params else {}
    response = await client.get("/bits/leaderboard", params=query)
    return GetBitsLeaderboardResponse.model_validate(response)


async def get_cheermotes(
    client: "TwitchHTTPClient",
    params: GetCheermotesRequest | None = None,
) -> TwitchResponse[Cheermote]:
    """Get cheermotes for a channel or global cheermotes."""
    query = params.model_dump(exclude_none=True) if params else {}
    response = await client.get("/bits/cheermotes", params=query)
    return TwitchResponse[Cheermote].model_validate(response)


async def get_extension_transactions(
    client: "TwitchHTTPClient",
    params: GetExtensionTransactionsRequest,
) -> TwitchResponse[ExtensionTransaction]:
    """Get extension transactions.

    Requires extension developer or extension owner.
    """
    query = params.model_dump(exclude_none=True)
    response = await client.get("/extensions/transactions", params=query)
    return TwitchResponse[ExtensionTransaction].model_validate(response)



"""Hype Train endpoints."""

from typing import TYPE_CHECKING

from twitch_sdk.schemas.base import TwitchResponse
from twitch_sdk.schemas.hype_train import GetHypeTrainEventsRequest, HypeTrainEvent

if TYPE_CHECKING:
    from twitch_client import TwitchHTTPClient


async def get_hype_train_events(
    client: "TwitchHTTPClient",
    params: GetHypeTrainEventsRequest,
) -> TwitchResponse[HypeTrainEvent]:
    """Get hype train events for a broadcaster.

    Requires channel:read:hype_train scope.
    """
    query = params.model_dump(exclude_none=True)
    response = await client.get("/hypetrain/events", params=query)
    return TwitchResponse[HypeTrainEvent].model_validate(response)
